unit MaskImageBtn;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, LMessages,
  StdCtrls, FPImage, DateUtils, ExtCtrls;

type
  TMarigns=class(TPersistent)
  private
   FLeft,FTop,FRight,FBottom:integer;
   FOnChange:TNotifyEvent;
   Procedure SetLeft(Value:Integer);
   Procedure SetRight(Value:Integer);
   Procedure SetTop(Value:Integer);
   Procedure SetBottom(Value:Integer);
  public
   Constructor Create;
   Procedure Assign(Source:TPersistent);
   Property OnChange:TNotifyEvent read fOnChange write fOnChange;
  published
   Property Left:integer read FLeft write SetLeft default 0;
   Property Top:integer read FTop write SetTop default 0;
   Property Right:integer read FRight write SetRight default 0;
   Property Bottom:integer read FBottom write SetBottom default 0;
  end;

  { TMaskImageBtn }

  TMaskImageBtn = class(TCustomControl)
  private
   FBackGround:TBitmap;
   FGliph:TBitmap;
   FPicture:TBitmap;
   FFont:TFont;
   FDownGliph:TBitmap;
   FDownHighlightGliph:TBitmap;
   FHighlightGliph:TBitmap;
   FColor:TColor;
   FCaption:TCaption;
   FEnabled:Boolean;
   FImageMarigns:TMarigns;
   FGroupIndex:integer;
   FDown,FMDown,FHighlight,FFixDown:boolean;
   FOnMouseEnter,FOnMouseLeave:TNotifyEvent;
   Procedure SetEnabled(Value:boolean);
   Procedure SetDown(Value:boolean);
   Procedure SetColor(Sender:TColor);
   Procedure SetCaption(Sender:TCaption);
   Procedure SetGliph(Sender: TBitmap);
   Procedure SetPicture(Sender: TBitmap);
   Procedure SetDownGliph(Sender: TBitmap);
   Procedure SetDownHighlightGliph(Sender: TBitmap);
   Procedure SetHighlightGliph(Sender: TBitmap);
   Procedure SetBackGround(Sender: TBitmap);
   Procedure SetFont(AFont:TFont);
   Procedure SetImageMarigns(Value:TMarigns);
   Procedure SetGroupIndex(Value: integer);
   Procedure ImageMarignsChange(Sender:TObject);
  protected
   Procedure Paint; override;
   Procedure MouseLeave(var msg: TLMessage); Message CM_MouseLeave;
   Procedure MouseEnter(var msg: TLMessage); Message CM_MouseEnter;
   Procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   Procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    { Protected declarations }
  public
   Constructor Create(AOwner:TComponent); override;
   Destructor Destroy; override;
   Procedure Click; override;
   procedure Assign(Source: TPersistent); override;
  published
   Property Align;
   Property Hint;
   Property ShowHint;
   Property ParentShowHint;
   Property Visible;
   Property Anchors;
   Property GroupIndex: integer read FGroupIndex write SetGroupIndex default -1;
   Property Down:boolean read FFixDown write SetDown default false;
   Property Gliph:TBitmap read FGliph Write SetGliph;
   Property Picture:TBitmap read FPicture Write SetPicture;
   Property DownGliph:TBitmap read FDownGliph Write SetDownGliph;
   Property DownHighlightGliph:TBitmap read FDownHighlightGliph Write SetDownHighlightGliph;
   Property HighlightGliph:TBitmap read FHighlightGliph Write SetHighlightGliph;
   Property BackGround:TBitmap read FBackGround Write SetBackGround;
   Property Color:TColor read FColor write SetColor default clBtnFace;
   Property Caption:TCaption read FCaption write SetCaption;
   Property Enabled:boolean read FEnabled write SetEnabled default true;
   Property Font:TFont read FFont write SetFont;
   Property ImageMarigns:TMarigns read FImageMarigns write SetImageMarigns;
   Property Cursor;
   Property OnMouseEnter: TNotifyEvent read FOnMouseEnter Write FOnMouseEnter;
   Property OnMouseLeave: TNotifyEvent read FOnMouseLeave Write FOnMouseLeave;
   Property OnMouseDown;
   Property OnMouseUp;
   Property OnKeyPress;
   Property OnKeyDown;
   Property OnKeyUp;
   Property OnClick;
   Property OnDblClick;
    { Published declarations }
  end;

  { TMaskImgCheckBox }

  TMaskImgCheckBox = class(TGraphicControl)
  private
   FAllowGrayed: Boolean;
   FState: TCheckBoxState;
   FBackGround:TBitmap;
   FGliph:TBitmap;
   FHighlightGliph:TBitmap;
   FGrayGliph:TBitmap;
   FHighlightGrayGliph:TBitmap;
   FCheckGliph:TBitmap;
   FHighlightCheckGliph:TBitmap;
   FEnabled:boolean;
   FFont:TFont;
   FColor:TColor;
   FCaption:TCaption;
   FDown,FMDown,FHighlight:boolean;
   FOnMouseEnter,FOnMouseLeave:TNotifyEvent;
   procedure SetState(Value: TCheckBoxState);
   function GetChecked: Boolean;
   procedure SetEnabled(Value:boolean);
   procedure SetChecked(Value: Boolean);
   Procedure SetColor(Sender:TColor);
   Procedure SetCaption(Sender:TCaption);
   Procedure SetGliph(Sender: TBitmap);
   Procedure SetGrayGliph(Sender: TBitmap);
   Procedure SetCheckGliph(Sender: TBitmap);
   Procedure SetHighlightGliph(Sender: TBitmap);
   Procedure SetHighlightGrayGliph(Sender: TBitmap);
   Procedure SetHighlightCheckGliph(Sender: TBitmap);
   Procedure SetBackGround(Sender: TBitmap);
   Procedure SetFont(AFont:TFont);
   Procedure FontChange(Sender:TObject);
   Procedure DoAutoSize; override;
  protected
   Procedure Click; override;
   Procedure Paint; override;
   Procedure MouseLeave(var msg: TLMessage); Message CM_MouseLeave;
   Procedure MouseEnter(var msg: TLMessage); Message CM_MouseEnter;
   Procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   Procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    { Protected declarations }
  public
   Constructor Create(AOwner:TComponent); override;
   procedure Assign(Source: TPersistent); override;
  published
   Property Align;
   Property Hint;
   Property ShowHint;
   Property ParentShowHint;
   Property Visible;
   property AutoSize default true;
   property AllowGrayed: Boolean read FAllowGrayed write FAllowGrayed default false;
   property State: TCheckBoxState read FState write SetState;
   Property Checked:boolean read GetChecked write SetChecked;
   Property Anchors;
   Property Enabled:boolean read fEnabled write SetEnabled default true;
   Property Gliph:TBitmap read FGliph Write SetGliph;
   Property CheckGliph:TBitmap read FCheckGliph Write SetCheckGliph;
   Property GrayGliph:TBitmap read FGrayGliph Write SetGrayGliph;
   Property HighlightGliph:TBitmap read FHighlightGliph Write SetHighlightGliph;
   Property HighlightCheckGliph:TBitmap read FHighlightCheckGliph Write SetHighlightCheckGliph;
   Property HighlightGrayGliph:TBitmap read FHighlightGrayGliph Write SetHighlightGrayGliph;
   Property BackGround:TBitmap read FBackGround Write SetBackGround;
   Property Color:TColor read FColor write SetColor default clBtnFace;
   Property Caption:TCaption read FCaption write SetCaption;
   Property Font:TFont read FFont write SetFont;
   Property OnMouseEnter: TNotifyEvent read FOnMouseEnter Write FOnMouseEnter;
   Property OnMouseLeave: TNotifyEvent read FOnMouseLeave Write FOnMouseLeave;
   Property OnMouseDown;
   Property OnMouseUp;
   Property OnClick;
   Property OnDblClick;
    { Published declarations }
  end;

  { TMaskImgRadioButton }

  TMaskImgRadioButton = class(TGraphicControl)
  private
   FGroupIndex:integer;
   FChecked: boolean;
   FBackGround:TBitmap;
   FGliph:TBitmap;
   FHighlightGliph:TBitmap;
   FCheckGliph:TBitmap;
   FHighlightCheckGliph:TBitmap;
   FEnabled:boolean;
   FFont:TFont;
   FColor:TColor;
   FCaption:TCaption;
   FDown,FMDown,FHighlight:boolean;
   FOnMouseEnter,FOnMouseLeave:TNotifyEvent;
   procedure SetEnabled(Value:boolean);
   procedure SetChecked(Value: Boolean);
   Procedure SetColor(Sender:TColor);
   Procedure SetCaption(Sender:TCaption);
   Procedure SetGliph(Sender: TBitmap);
   Procedure SetCheckGliph(Sender: TBitmap);
   Procedure SetHighlightGliph(Sender: TBitmap);
   Procedure SetHighlightCheckGliph(Sender: TBitmap);
   Procedure SetBackGround(Sender: TBitmap);
   Procedure SetFont(AFont:TFont);
   Procedure FontChange(Sender:TObject);
   Procedure DoAutoSize; override;
  protected
   Procedure Click; override;
   Procedure Paint; override;
   Procedure MouseLeave(var msg: TLMessage); Message CM_MouseLeave;
   Procedure MouseEnter(var msg: TLMessage); Message CM_MouseEnter;
   Procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   Procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    { Protected declarations }
  public
   Constructor Create(AOwner:TComponent); override;
   procedure Assign(Source: TPersistent); override;
  published
   Property Align;
   Property Hint;
   Property ShowHint;
   Property ParentShowHint;
   Property Visible;
   property AutoSize default true;
   Property Anchors;
   Property Enabled:boolean read fEnabled write SetEnabled default true;
   Property Checked:boolean read FChecked write SetChecked;
   Property Gliph:TBitmap read FGliph Write SetGliph;
   Property GroupIndex:integer read FGroupIndex write FGroupIndex;
   Property CheckGliph:TBitmap read FCheckGliph Write SetCheckGliph;
   Property HighlightGliph:TBitmap read FHighlightGliph Write SetHighlightGliph;
   Property HighlightCheckGliph:TBitmap read FHighlightCheckGliph Write SetHighlightCheckGliph;
   Property BackGround:TBitmap read FBackGround Write SetBackGround;
   Property Color:TColor read FColor write SetColor default clBtnFace;
   Property Caption:TCaption read FCaption write SetCaption;
   Property Font:TFont read FFont write SetFont;
   Property OnMouseEnter: TNotifyEvent read FOnMouseEnter Write FOnMouseEnter;
   Property OnMouseLeave: TNotifyEvent read FOnMouseLeave Write FOnMouseLeave;
   Property OnMouseDown;
   Property OnMouseUp;
   Property OnClick;
   Property OnDblClick;
    { Published declarations }
  end;

  { TMaskImageScrollBar }

  TMaskImageScrollBar = class(TCustomControl)
  private
   FPosition,FCount:double;
   FSliderBackGround:TBitmap;
   FBackGround:TBitmap;
   FGliph:TBitmap;
   FPicture:TBitmap;
   FDownGliph:TBitmap;
   FHighlightGliph:TBitmap;
   FColor,FSliderColor:TColor;
   FFont:TFont;
   FCaption:TCaption;
   FEnabled:Boolean;
   FSliderMarigns,FImageMarigns,FSliderImageMarigns:TMarigns;
   fOnScroll:TNotifyEvent;
   FMinSliderWidth:integer;
   FDown,FHighlight:boolean;
   FKind: TScrollBarKind;
   FMouseDownPos:TPoint;
   FCanUpdate:boolean;
   FLastUpdate:TDateTime;
   FTimer:TTimer;
   procedure SetCaption(const AValue: TCaption);
   procedure SetEnabled(const AValue: boolean);
   procedure SetFont(const AValue: TFont);
   procedure SetKind(Value:TScrollBarKind);
   Procedure SetColor(Sender:TColor);
   Procedure SetSliderColor(Sender:TColor);
   Procedure SetMinSliderWidth(Value:integer);
   Procedure SetPosition(Value:double);
   Procedure SetCount(Value:double);
   Procedure SetGliph(Sender: TBitmap);
   Procedure SetDownGliph(Sender: TBitmap);
   Procedure SetHighlightGliph(Sender: TBitmap);
   Procedure SetSliderBackGround(Sender: TBitmap);
   Procedure SetBackGround(Sender: TBitmap);
   Procedure SetImageMarigns(Value:TMarigns);
   Procedure SetSliderImageMarigns(Value:TMarigns);
   Procedure SetSliderMarigns(Value:TMarigns);
   Procedure ImageMarignsChange(Sender:TObject);
   Procedure TimerEvent(Sender:TObject);
   Function GetSliderRect:TRect;
  protected
   Procedure Paint; override;
   procedure MouseEnter; override;
   procedure MouseLeave; override;
   procedure MouseMove(Shift: TShiftState; X,Y: Integer); override;
   Procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   Procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    { Protected declarations }
  public
   Constructor Create(AOwner:TComponent); override;
   Destructor Destroy; override;
   procedure Assign(Source: TPersistent); override;
  published
   Property Align;
   Property Hint;
   Property ShowHint;
   Property ParentShowHint;
   Property Visible;
   Property Anchors;
   Property Kind: TScrollBarKind read FKind write SetKind default sbHorizontal;
   Property SliderGliph:TBitmap read FGliph Write SetGliph;
   Property SliderDownGliph:TBitmap read FDownGliph Write SetDownGliph;
   Property SliderHighlightGliph:TBitmap read FHighlightGliph Write SetHighlightGliph;
   Property BackGround:TBitmap read FBackGround Write SetBackGround;
   Property SliderBackGround:TBitmap read FSliderBackGround Write SetBackGround;
   Property SliderColor:TColor read FSliderColor write SetSliderColor default clBlack;
   Property Color:TColor read FColor write SetColor default clWhite;
   Property Enabled:boolean read FEnabled write SetEnabled default true;
   Property ImageMarigns:TMarigns read FImageMarigns write SetImageMarigns;
   Property SliderImageMarigns:TMarigns read FSliderImageMarigns write SetSliderImageMarigns;
   Property SliderMarigns:TMarigns read FSliderMarigns write SetSliderMarigns;
   Property MinSliderWidth:integer read FMinSliderWidth write SetMinSliderWidth;
   Property Position:double read FPosition write SetPosition;
   Property Caption:TCaption read FCaption write SetCaption;
   Property Font:TFont read FFont write SetFont;
   Property Count:double read FCount write SetCount;
   Property Cursor;
   Property OnScroll: TNotifyEvent read FOnScroll Write FOnScroll;
   Property OnMouseEnter;
   Property OnMouseLeave;
   Property OnMouseDown;
   Property OnMouseUp;
   Property OnClick;
   Property OnDblClick;
   Property OnKeyPress;
   Property OnKeyDown;
   Property OnKeyUp;
    { Published declarations }
  end;

function IsPointInRect(aPoint:TPoint;aRect:TRect):boolean;

procedure Register;

implementation

Constructor TMarigns.Create;
begin
 FLeft:=0;
 FRight:=0;
 FTop:=0;
 FBottom:=0;
 FOnChange:=nil;
end;

Procedure TMarigns.SetLeft(Value:Integer);
begin
 FLeft:=value;
 if FOnChange<>nil then FOnChange(self);
end;

Procedure TMarigns.SetTop(Value:Integer);
begin
 FTop:=value;
 if FOnChange<>nil then FOnChange(self);
end;

Procedure TMarigns.SetRight(Value:Integer);
begin
 FRight:=value;
 if FOnChange<>nil then FOnChange(self);
end;

Procedure TMarigns.SetBottom(Value:Integer);
begin
 FBottom:=value;
 if FOnChange<>nil then FOnChange(self);
end;

Procedure TMarigns.Assign(Source:TPersistent);
begin
 if Source is TMarigns then
 begin
  FLeft:=(Source as TMarigns).Left;
  FTop:=(Source as TMarigns).Top;
  FRight:=(Source as TMarigns).Right;
  FBottom:=(Source as TMarigns).Bottom;
 end;
end;

Function IsImageAssigned(aBitmap:TRasterImage):boolean;
begin
 result:=(aBitmap.Width<>0) and (aBitmap.Height<>0);
end;

Function IsPointInRect(aPoint:TPoint;aRect:TRect):boolean;
begin
 result:=(aPoint.X>=aRect.Left) and (aPoint.X<=aRect.Right) and (aPoint.Y>=aRect.Top) and (aPoint.Y<=aRect.Bottom);
end;

procedure DrawImageWithBorder(aDest:TBitmap;aSrc:TBitmap;aDestRect:TRect;aMarigns:TMarigns);
var
 aRect,FRect,LRect:TRect;
begin
 with aDest do
 begin
  aRect:=Rect(0,0,aSrc.Width,aSrc.Height);
  {left-top}
  FRect:=aRect;
  FRect.Bottom:=aMarigns.Top;
  FRect.Right:=aMarigns.Left;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Bottom:=FRect.Top+aMarigns.Top;
  FRect.Right:=FRect.left+aMarigns.Left;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {left-bottom}
  FRect:=aRect;
  FRect.Top:=FRect.Bottom-aMarigns.Bottom;
  FRect.Right:=aMarigns.Left;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Top:=FRect.Bottom-aMarigns.Bottom;
  FRect.Right:=FRect.left+aMarigns.Left;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {right-top}
  FRect:=aRect;
  FRect.Bottom:=aMarigns.Top;
  FRect.Left:=FRect.Right-aMarigns.Right;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Bottom:=FRect.Top+aMarigns.Top;
  FRect.Left:=FRect.Right-aMarigns.Right;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {right-bottom}
  FRect:=aRect;
  FRect.Top:=FRect.Bottom-aMarigns.Bottom;
  FRect.Left:=FRect.Right-aMarigns.Right;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Top:=FRect.Bottom-aMarigns.Bottom;
  FRect.Left:=FRect.Right-aMarigns.Right;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {left}
  FRect:=aRect;
  FRect.Top:=aMarigns.Top;
  FRect.Bottom:=FRect.Bottom-aMarigns.Bottom;
  FRect.Right:=aMarigns.Left;
  if (FRect.Bottom-FRect.Top)=0 then FRect.Bottom:=FRect.Bottom+1;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Top:=FRect.Top+aMarigns.Top;
  FRect.Bottom:=FRect.Bottom-aMarigns.Bottom;
  FRect.Right:=FRect.Left+aMarigns.Left;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {right}
  FRect:=aRect;
  FRect.Top:=aMarigns.Top;
  FRect.Bottom:=FRect.Bottom-aMarigns.Bottom;
  FRect.Left:=FRect.Right-aMarigns.Right;
  if (FRect.Bottom-FRect.Top)=0 then FRect.Bottom:=FRect.Bottom+1;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Top:=FRect.Top+aMarigns.Top;
  FRect.Bottom:=FRect.Bottom-aMarigns.Bottom;
  FRect.Left:=FRect.Right-aMarigns.Right;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {top}
  FRect:=aRect;
  FRect.Right:=FRect.Right-aMarigns.Right;
  FRect.Left:=aMarigns.Left;
  FRect.Bottom:=aMarigns.Top;
  if (FRect.Right-FRect.Left)=0 then FRect.Right:=FRect.Right+1;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Right:=FRect.Right-aMarigns.Right;
  FRect.Left:=FRect.Left+aMarigns.Left;
  FRect.Bottom:=FRect.Top+aMarigns.Top;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {bottom}
  FRect:=aRect;
  FRect.Right:=FRect.Right-aMarigns.Right;
  FRect.Left:=aMarigns.Left;
  FRect.Top:=FRect.Bottom-aMarigns.Bottom;
  if (FRect.Right-FRect.Left)=0 then FRect.Right:=FRect.Right+1;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Right:=FRect.Right-aMarigns.Right;
  FRect.Left:=FRect.Left+aMarigns.Left;
  FRect.Top:=FRect.Bottom-aMarigns.Bottom;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
  {center}
  FRect:=aRect;
  FRect.Right:=FRect.Right-aMarigns.Right;
  FRect.Left:=aMarigns.Left;
  FRect.Top:=aMarigns.Top;
  FRect.Bottom:=FRect.Bottom-aMarigns.Bottom;
  if (FRect.Right-FRect.Left)=0 then FRect.Right:=FRect.Right+1;
  if (FRect.Bottom-FRect.Top)=0 then FRect.Bottom:=FRect.Bottom+1;
  LRect:=FRect;
  FRect:=aDestRect;
  FRect.Right:=FRect.Right-aMarigns.Right;
  FRect.Left:=FRect.Left+aMarigns.Left;
  FRect.Top:=FRect.Top+aMarigns.Top;
  FRect.Bottom:=FRect.Bottom-aMarigns.Bottom;
  Canvas.CopyRect(FRect,aSrc.Canvas,LRect);
 end;
end;

Constructor TMaskImageBtn.Create(AOwner:TComponent);
begin
 Inherited Create(AOwner);
 FGroupIndex:=-1;
 FColor:=clBtnFace;
 FCaption:=Name;
 FBackGround:=TBitmap.Create;
 FPicture:=TBitmap.Create;
 FGliph:=TBitmap.Create;
 FDownGliph:=TBitmap.Create;
 FDownHighlightGliph:=TBitmap.Create;
 FHighlightGliph:=TBitmap.Create;
 FEnabled:=true;
 FDown:=false;
 FMDown:=false;
 FHighlight:=false;
 FFont:=TFont.Create;
 FFont.OnChange:=@ImageMarignsChange;
 FImageMarigns:=TMarigns.Create;
 FImageMarigns.OnChange:=@ImageMarignsChange;
 Height:=23;
 Width:=75;
 FFixDown:=False;
 {$IFDEF WINDOWS}
 DoubleBuffered:=true;
 {$ENDIF}
end;

Destructor TMaskImageBtn.Destroy;
begin
 FBackGround.Free;
 FPicture.Free;
 FGliph.Free;
 FDownGliph.Free;
 FDownHighlightGliph.Free;
 FHighlightGliph.Free;
 Inherited Destroy;
end;

Procedure TMaskImageBtn.Click;
begin
 if FEnabled then inherited Click;
end;

procedure TMaskImageBtn.Assign(Source: TPersistent);
begin
  //inherited Assign(Source);
  FBackGround.Assign((Source as TMaskImageBtn).FBackGround);
  FGliph.Assign((Source as TMaskImageBtn).FGliph);
  FPicture.Assign((Source as TMaskImageBtn).FPicture);
  FFont.Assign((Source as TMaskImageBtn).FFont);
  FDownGliph.Assign((Source as TMaskImageBtn).FDownGliph);
  FDownHighlightGliph.Assign((Source as TMaskImageBtn).FDownHighlightGliph);
  FHighlightGliph.Assign((Source as TMaskImageBtn).FHighlightGliph);
  FColor:=(Source as TMaskImageBtn).FColor;
  FCaption:=(Source as TMaskImageBtn).FCaption;
  FEnabled:=(Source as TMaskImageBtn).FEnabled;
  FImageMarigns.Assign((Source as TMaskImageBtn).FImageMarigns);
  FGroupIndex:=(Source as TMaskImageBtn).FGroupIndex;
  FDown:=(Source as TMaskImageBtn).FDown;
end;

Procedure TMaskImageBtn.SetGroupIndex(Value: integer);
var
 i:integer;
begin
 FGroupIndex:=value;
 if Parent=nil then exit;
 if FFixDown then
 begin
  for i:=0 to Parent.ControlCount-1 do
   if Parent.Controls[i] is TMaskImageBtn then
    if (Parent.Controls[i]<>self) and ((Parent.Controls[i] as TMaskImageBtn).GroupIndex=FGroupIndex) and ((Parent.Controls[i] as TMaskImageBtn).Down) then (Parent.Controls[i] as TMaskImageBtn).Down:=false;
 end;
end;

Procedure TMaskImageBtn.SetEnabled(Value:boolean);
begin
 FEnabled:=Value;
 repaint;
end;

Procedure TMaskImageBtn.SetDown(Value:boolean);
var
 i:integer;
begin
 if FGroupIndex>-1 then
 begin
  FFixDown:=Value;
  if Parent=nil then exit;
  if FFixDown then
  begin
   for i:=0 to Parent.ControlCount-1 do
    if Parent.Controls[i] is TMaskImageBtn then
     if (Parent.Controls[i]<>self) and ((Parent.Controls[i] as TMaskImageBtn).GroupIndex=FGroupIndex) and ((Parent.Controls[i] as TMaskImageBtn).Down) then (Parent.Controls[i] as TMaskImageBtn).Down:=false;
  end;
  repaint;
 end;
end;

Procedure TMaskImageBtn.ImageMarignsChange(Sender:TObject);
begin
 repaint;
end;

Procedure TMaskImageBtn.MouseLeave(var msg: TLMessage);
begin
 if csDesigning in ComponentState then exit;
 if FEnabled then
 begin
  FDown:=False;
  FHighlight:=False;
 end;
 if Assigned(FOnMouseLeave) then OnMouseLeave(self);
 repaint;
end;

Procedure TMaskImageBtn.MouseEnter(var msg: TLMessage);
begin
 if csDesigning in ComponentState then exit;
 if FEnabled then
 begin
  If FMDown then FDown:=True;
  FHighlight:=true;
 end;
 if Assigned(FOnMouseEnter) then OnMouseEnter(self);
 repaint;
end;

Procedure TMaskImageBtn.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if FEnabled then
 begin
  FMDown:=True;
  FDown:=True;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 Repaint;
end;

Procedure TMaskImageBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 fflag:boolean;
 i:integer;
begin
 if FEnabled then
 begin
  FMDown:=False;
  FDown:=False;
  fflag:=false;
  if FGroupIndex<>-1 then
  begin
   for i:=0 to Parent.ControlCount-1 do
    if Parent.Controls[i] is TMaskImageBtn then
     if (Parent.Controls[i]<>self) and ((Parent.Controls[i] as TMaskImageBtn).GroupIndex=FGroupIndex) then fflag:=true;
  end;
  if fflag then
   Down:=true
  else
   Down:=not FFixDown;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 Repaint;
end;

Procedure TMaskImageBtn.Paint;
var
 frect,arect,lrect:TRect;
 aoffset:integer;
 fbmp,ftmp:TBitmap;
begin
 fbmp:=TBitmap.Create;
 fbmp.Width:=width;
 fbmp.Height:=Height;
 if FColor=clNone then
 begin
  fbmp.Canvas.Brush.Style:=bsClear;
  fbmp.Transparent:=true;
 end else fbmp.Canvas.Brush.Style:=bsSolid;
 fbmp.Canvas.Brush.Color:=FColor;
 fbmp.Canvas.FillRect(ClientRect);
 if IsImageAssigned(FBackGround) then
  fbmp.Canvas.StretchDraw(ClientRect,FBackGround);
 FTmp:=nil;
 if FDown then
 begin
  if FFixDown then
  begin
   if IsImageAssigned(FGliph) then
    FTmp:=FGliph;
  end else
  begin
   if IsImageAssigned(FDownGliph) then
    FTmp:=FDownGliph;
  end;
 end
 else if FHighlight then
 begin
  if FFixDown then
  begin
   if IsImageAssigned(FDownHighlightGliph) then
    FTmp:=FDownHighlightGliph;
  end else
  begin
   if IsImageAssigned(FHighlightGliph) then
   FTmp:=FHighlightGliph;
  end;
 end
 else
 begin
  if FFixDown then
  begin
   if IsImageAssigned(FDownGliph) then
    FTmp:=FDownGliph;
  end else
  begin
   if IsImageAssigned(FGliph) then
    FTmp:=FGliph;
  end;
 end;
 if FTmp<>nil then
  DrawImageWithBorder(fbmp,ftmp,ClientRect,FImageMarigns);
 fbmp.Mask(clDefault);
 Canvas.Draw(0,0,fbmp);
 Canvas.Brush.Style:=bsClear;
 Canvas.Font:=FFont;
 aoffset:=0;
 if (FPicture.Width<>0) and (FCaption<>'') then aoffset:=2;
 if not FEnabled then Canvas.Font.Color:=RGBToColor(Red(ColorToRGB(Canvas.Font.Color)) shr 1,Green(ColorToRGB(Canvas.Font.Color)) shr 1,Blue(ColorToRGB(Canvas.Font.Color)) shr 1);
 if FDown xor FFixDown then
 begin
  Canvas.Draw(width div 2 - canvas.TextWidth(FCaption) div 2+1-aoffset div 2-FPicture.Width div 2,Height div 2 - FPicture.Height div 2 +1,FPicture);
  Canvas.TextRect(ClientRect,aoffset div 2+FPicture.Width div 2+width div 2 - canvas.TextWidth(FCaption) div 2+1,Height div 2 - canvas.TextHeight(FCaption) div 2+1,FCaption);
 end else
 begin
  Canvas.Draw(width div 2 - canvas.TextWidth(FCaption) div 2-aoffset div 2-FPicture.Width div 2,Height div 2 - FPicture.Height div 2,FPicture);
  Canvas.TextRect(ClientRect,aoffset div 2+FPicture.Width div 2+width div 2 - canvas.TextWidth(FCaption) div 2,Height div 2 - canvas.TextHeight(FCaption) div 2,FCaption);
 end;
 fbmp.free;
end;

Procedure TMaskImageBtn.SetImageMarigns(Value:TMarigns);
begin
 if FImageMarigns=Value then exit;
 FImageMarigns.Assign(Value);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetFont(AFont:TFont);
begin
 FFont.Assign(AFont);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetColor(Sender:TColor);
begin
 FColor:=sender;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetCaption(Sender:TCaption);
begin
 FCaption:=sender;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetGliph(Sender: TBitmap);
begin
 FGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetPicture(Sender: TBitmap);
begin
 FPicture.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetDownGliph(Sender: TBitmap);
begin
 FDownGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetDownHighlightGliph(Sender: TBitmap);
begin
 FDownHighlightGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetHighlightGliph(Sender: TBitmap);
begin
 FHighlightGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageBtn.SetBackGround(Sender: TBitmap);
begin
 FBackGround.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

{TMaskImgCheckBox}

Constructor TMaskImgCheckBox.Create(AOwner:TComponent);
begin
 Inherited Create(AOwner);
 FColor:=clBtnFace;
 FCaption:=Name;
 FBackGround:=TBitmap.Create;
 FGliph:=TBitmap.Create;
 FGrayGliph:=TBitmap.Create;
 FCheckGliph:=TBitmap.Create;
 FHighlightGliph:=TBitmap.Create;
 FHighlightGrayGliph:=TBitmap.Create;
 FHighlightCheckGliph:=TBitmap.Create;
 FDown:=false;
 FMDown:=false;
 FHighlight:=false;
 FEnabled:=true;
 FFont:=TFont.Create;
 FFont.OnChange:=@FontChange;
 FCaption:=Name;
 AutoSize:=true;
 Width:=100;
 Height:=16;
end;

procedure TMaskImgCheckBox.Assign(Source: TPersistent);
begin
  //inherited Assign(Source);
  FAllowGrayed:=(Source as TMaskImgCheckBox).FAllowGrayed;
  FState:=(Source as TMaskImgCheckBox).FState;
  FBackGround.Assign((Source as TMaskImgCheckBox).FBackGround);
  FGliph.Assign((Source as TMaskImgCheckBox).FGliph);
  FHighlightGliph.Assign((Source as TMaskImgCheckBox).FHighlightGliph);
  FGrayGliph.Assign((Source as TMaskImgCheckBox).FGrayGliph);
  FHighlightGrayGliph.Assign((Source as TMaskImgCheckBox).FHighlightGrayGliph);
  FCheckGliph.Assign((Source as TMaskImgCheckBox).FCheckGliph);
  FHighlightCheckGliph.Assign((Source as TMaskImgCheckBox).FHighlightCheckGliph);
  FFont.Assign((Source as TMaskImgCheckBox).FFont);
  FEnabled:=(Source as TMaskImgCheckBox).FEnabled;
  FColor:=(Source as TMaskImgCheckBox).FColor;
  FCaption:=(Source as TMaskImgCheckBox).FCaption;
  FDown:=(Source as TMaskImgCheckBox).FDown;
end;

Procedure TMaskImgCheckBox.FontChange(Sender:TObject);
begin
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.MouseLeave(var msg: TLMessage);
begin
 if csDesigning in ComponentState then exit;
 if FEnabled then
 begin
  FDown:=False;
  FHighlight:=False;
 end;
 if Assigned(FOnMouseLeave) then OnMouseLeave(self);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.MouseEnter(var msg: TLMessage);
begin
 if csDesigning in ComponentState then exit;
 if FEnabled then
 begin
  If FMDown then FDown:=True;
  FHighlight:=true;
 end;
 if Assigned(FOnMouseEnter) then OnMouseEnter(self);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if FEnabled then
 begin
  FMDown:=True;
  FDown:=True;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 if Canvas.Locked then exit;
 Repaint;
end;

Procedure TMaskImgCheckBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if FEnabled then
 begin
  FMDown:=False;
  FDown:=False;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 if Canvas.Locked then exit;
 Repaint;
end;

Procedure TMaskImgCheckBox.Paint;
var
 frect,arect:TRect;
 fbmp:TBitmap;
// ms:TMemoryStream;
 i,r:integer;
 cl:TFPColor;
begin
 fbmp:=TBitmap.Create;
 fbmp.Width:=width;
 fbmp.Height:=Height;
// ms:=TMemoryStream.Create;
 Canvas.Brush.Color:=FColor;
 if FColor=clNone then
 begin
  Canvas.Brush.Style:=bsClear;
  if FHighlight then
  begin
   case FState of
    cbUnChecked:Canvas.Brush.Color:=FHighlightGliph.Canvas.Pixels[0,FHighlightGliph.Height-1];
    cbChecked:Canvas.Brush.Color:=FHighlightCheckGliph.Canvas.Pixels[0,FHighlightCheckGliph.Height-1];
    cbGrayed:Canvas.Brush.Color:=FHighlightGrayGliph.Canvas.Pixels[0,FHighlightGrayGliph.Height-1];
   end;
  end else
   case FState of
    cbUnChecked:Canvas.Brush.Color:=FGliph.Canvas.Pixels[0,FGliph.Height-1];
    cbChecked:Canvas.Brush.Color:=FCheckGliph.Canvas.Pixels[0,FCheckGliph.Height-1];
    cbGrayed:Canvas.Brush.Color:=FGrayGliph.Canvas.Pixels[0,FGrayGliph.Height-1];
   end;
  fbmp.Transparent:=true;
  fbmp.Canvas.Brush.Style:=bsSolid;
  fbmp.Canvas.Brush.Color:=Canvas.Brush.Color;
  fbmp.Canvas.FillRect(fbmp.Canvas.ClipRect);
 end else Canvas.Brush.Style:=bsSolid;
 Canvas.FillRect(ClientRect);
 if IsImageAssigned(FBackGround) then Canvas.StretchDraw(ClientRect,FBackGround);
 if FHighlight then
 begin
  with fbmp do
  begin
   case FState of
    cbUnChecked:aRect:=Rect(0,0,FHighlightGliph.Width,FHighlightGliph.Height);
    cbChecked:aRect:=Rect(0,0,FHighlightCheckGliph.Width,FHighlightCheckGliph.Height);
    cbGrayed:aRect:=Rect(0,0,FHighlightGrayGliph.Width,FHighlightGrayGliph.Height);
   end;
   FRect:=ClientRect;
   FRect.Right:=aRect.Right;
   FRect.Left:=0;
   FRect.Top:=(FRect.Bottom div 2)-(aRect.Bottom div 2);
   FRect.Bottom:=(FRect.Bottom div 2)+(aRect.Bottom div 2)+(aRect.Bottom mod 2);
   case FState of
    cbUnChecked:Canvas.CopyRect(FRect,FHighlightGliph.Canvas,aRect);
    cbChecked:Canvas.CopyRect(FRect,FHighlightCheckGliph.Canvas,aRect);
    cbGrayed:Canvas.CopyRect(FRect,FHighlightGrayGliph.Canvas,aRect);
   end;
  end;
 end
 else
 begin
  with fbmp do
  begin
   case FState of
    cbUnChecked:aRect:=Rect(0,0,FGliph.Width,FGliph.Height);
    cbChecked:aRect:=Rect(0,0,FCheckGliph.Width,FCheckGliph.Height);
    cbGrayed:aRect:=Rect(0,0,FGrayGliph.Width,FGrayGliph.Height);
   end;
   FRect:=ClientRect;
   FRect.Right:=aRect.Right;
   FRect.Left:=0;
   FRect.Top:=(FRect.Bottom div 2)-(aRect.Bottom div 2);
   FRect.Bottom:=(FRect.Bottom div 2)+(aRect.Bottom div 2)+(aRect.Bottom mod 2);
   case FState of
    cbUnChecked:Canvas.CopyRect(FRect,FGliph.Canvas,aRect);
    cbChecked:Canvas.CopyRect(FRect,FCheckGliph.Canvas,aRect);
    cbGrayed:Canvas.CopyRect(FRect,FGrayGliph.Canvas,aRect);
   end;
  end;
 end;
 if not FEnabled then
 begin
  For i:=0 to fbmp.Canvas.Width do
  For r:=0 to fbmp.Canvas.Height do
  begin
   cl:=fbmp.Canvas.Colors[i,r];
   cl.blue:=cl.Blue div 2;
   cl.Red:=cl.Red div 2;
   cl.Green:=cl.Green div 2;
   fbmp.Canvas.Colors[i,r]:=cl;
  end;
 end;
 fbmp.Mask(clDefault);
// fbmp.SaveToStream(ms);
// ms.Position:=0;
// fbmp.LoadFromStream(ms);
// ms.Position:=0;
 Canvas.Draw(0,0,fbmp);
 Canvas.Brush.Style:=bsClear;
 Canvas.Font:=FFont;
 if not FEnabled then Canvas.Font.Color:=RGBToColor(Red(ColorToRGB(Canvas.Font.Color)) shr 1,Green(ColorToRGB(Canvas.Font.Color)) shr 1,Blue(ColorToRGB(Canvas.Font.Color)) shr 1);
 FRect:=ClientRect;
 if FHighlight then
 case FState of
  cbUnChecked:FRect.Left:=FHighlightGliph.Width;
  cbChecked:FRect.Left:=FHighlightCheckGliph.Width;
  cbGrayed:FRect.Left:=FHighlightGrayGliph.Width;
 end else
 case FState of
  cbUnChecked:FRect.Left:=FGliph.Width;
  cbChecked:FRect.Left:=FCheckGliph.Width;
  cbGrayed:FRect.Left:=FGrayGliph.Width;
 end;
 FRect.Left:=FRect.Left+5;
 Canvas.TextRect(FRect,FRect.Left,Height div 2 - canvas.TextHeight(FCaption) div 2,FCaption);
 fbmp.free;
// ms.free;
end;

Procedure TMaskImgCheckBox.SetFont(AFont:TFont);
begin
 FFont.Assign(AFont);
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetColor(Sender:TColor);
begin
 FColor:=sender;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetCaption(Sender:TCaption);
begin
 FCaption:=sender;
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetGliph(Sender: TBitmap);
begin
 FGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetGrayGliph(Sender: TBitmap);
begin
 FGrayGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetCheckGliph(Sender: TBitmap);
begin
 FCheckGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetHighlightGliph(Sender: TBitmap);
begin
 FHighlightGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetHighlightGrayGliph(Sender: TBitmap);
begin
 FHighlightGrayGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetHighlightCheckGliph(Sender: TBitmap);
begin
 FHighlightCheckGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetBackGround(Sender: TBitmap);
begin
 FBackGround.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.SetState(Value:TCheckBoxState);
begin
 if (Value<>cbGrayed) or FAllowGrayed then FState:=value;
 if Canvas.Locked then exit;
 repaint;
end;

function TMaskImgCheckBox.GetChecked:boolean;
begin
 result:=FState=cbChecked;
end;

Procedure TMaskImgCheckBox.SetChecked(Value:boolean);
begin
 if Value then FState:=cbChecked else FState:=cbUnchecked;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgCheckBox.DoAutoSize;
var
  ModifyWidth,
  ModifyHeight : Boolean;
  NewWidth: Integer;
  NewHeight: Integer;
  FRect:TRect;
begin
  If AutoSize and (not AutoSizing) then begin
//    AutoSizing := True;
    try
      ModifyWidth := Align in [alLeft,alRight,alNone];
      ModifyHeight := Align in [alTop,alBottom,alNone];
      NewWidth:=Width;
      NewHeight:=Height;
      Canvas.Font:=FFont;
      FRect:=ClientRect;
      if FHighlight then
      case FState of
       cbUnChecked:FRect.Left:=FHighlightGliph.Width;
       cbChecked:FRect.Left:=FHighlightCheckGliph.Width;
       cbGrayed:FRect.Left:=FHighlightGrayGliph.Width;
      end else
      case FState of
       cbUnChecked:FRect.Left:=FGliph.Width;
       cbChecked:FRect.Left:=FCheckGliph.Width;
       cbGrayed:FRect.Left:=FGrayGliph.Width;
      end;
      FRect.Left:=FRect.Left+5;
      If ModifyWidth and (FGliph.Width > 0) then
        NewWidth := FRect.Left+Canvas.TextWidth(FCaption)+2;
      If ModifyHeight and (FGliph.Height > 0) then;
        NewHeight := Canvas.TextHeight(FCaption)+2;
      if (NewWidth<>Width) or (NewHeight<>Height) then begin
        SetBounds(Left,Top,NewWidth,NewHeight);
        repaint;
      end;
    finally
//      AutoSizing := False;
    end;
  end;
end;

Procedure TMaskImgCheckBox.SetEnabled(Value:boolean);
begin
 FEnabled:=Value;
 if Canvas.Locked then exit;
 Repaint;
end;

Procedure TMaskImgCheckBox.Click;
begin
 inherited Click;
 if FEnabled then
 case FState of
  cbUnChecked:FState:=cbChecked;
  cbChecked:if FAllowGrayed then FState:=cbGrayed else FState:=cbUnchecked;
  cbGrayed:FState:=cbUnChecked;
 end;
end;

{TMaskImgRadioButton}

Constructor TMaskImgRadioButton.Create(AOwner:TComponent);
begin
 Inherited Create(AOwner);
 FColor:=clBtnFace;
 FBackGround:=TBitmap.Create;
 FGliph:=TBitmap.Create;
 FCheckGliph:=TBitmap.Create;
 FHighlightGliph:=TBitmap.Create;
 FHighlightCheckGliph:=TBitmap.Create;
 FDown:=false;
 FMDown:=false;
 FHighlight:=false;
 FEnabled:=true;
 FFont:=TFont.Create;
 FFont.OnChange:=@FontChange;
 FCaption:=Name;
 AutoSize:=true;
 Width:=100;
 Height:=16;
end;

procedure TMaskImgRadioButton.Assign(Source: TPersistent);
begin
  //inherited Assign(Source);
  FGroupIndex:=(Source as TMaskImgRadioButton).FGroupIndex;
  FChecked:=(Source as TMaskImgRadioButton).FChecked;
  FBackGround.Assign((Source as TMaskImgRadioButton).FBackGround);
  FGliph.Assign((Source as TMaskImgRadioButton).FGliph);
  FHighlightGliph.Assign((Source as TMaskImgRadioButton).FHighlightGliph);
  FCheckGliph.Assign((Source as TMaskImgRadioButton).FCheckGliph);
  FHighlightCheckGliph.Assign((Source as TMaskImgRadioButton).FHighlightCheckGliph);
  FEnabled:=(Source as TMaskImgRadioButton).FEnabled;
  FFont.Assign((Source as TMaskImgRadioButton).FFont);
  FColor:=(Source as TMaskImgRadioButton).FColor;
  FCaption:=(Source as TMaskImgRadioButton).FCaption;
  FDown:=(Source as TMaskImgRadioButton).FDown;
end;

Procedure TMaskImgRadioButton.FontChange(Sender:TObject);
begin
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.MouseLeave(var msg: TLMessage);
begin
 if csDesigning in ComponentState then exit;
 if FEnabled then
 begin
  FDown:=False;
  FHighlight:=False;
 end;
 if Assigned(FOnMouseLeave) then OnMouseLeave(self);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.MouseEnter(var msg: TLMessage);
begin
 if csDesigning in ComponentState then exit;
 if FEnabled then
 begin
  If FMDown then FDown:=True;
  FHighlight:=true;
 end;
 if Assigned(FOnMouseEnter) then OnMouseEnter(self);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if FEnabled then
 begin
  FMDown:=True;
  FDown:=True;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 if Canvas.Locked then exit;
 Repaint;
end;

Procedure TMaskImgRadioButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if FEnabled then
 begin
  FMDown:=False;
  FDown:=False;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 if Canvas.Locked then exit;
 Repaint;
end;

Procedure TMaskImgRadioButton.Paint;
var
 frect,arect:TRect;
 fbmp:TBitmap;
 ms:TMemoryStream;
 i,r:integer;
 cl:TFPColor;
begin
 fbmp:=TBitmap.Create;
 fbmp.Width:=width;
 fbmp.Height:=Height;
 ms:=TMemoryStream.Create;
 if FColor=clNone then
 begin
  Canvas.Brush.Style:=bsClear;
  fbmp.Transparent:=true;
 end else Canvas.Brush.Style:=bsSolid;
 Canvas.Brush.Color:=FColor;
 Canvas.FillRect(ClientRect);
 if IsImageAssigned(FBackGround) then Canvas.StretchDraw(ClientRect,FBackGround);
 if FHighlight then
 begin
  with fbmp do
  begin
   case FChecked of
    false:aRect:=Rect(0,0,FHighlightGliph.Width,FHighlightGliph.Height);
    true:aRect:=Rect(0,0,FHighlightCheckGliph.Width,FHighlightCheckGliph.Height);
   end;
   FRect:=ClientRect;
   FRect.Right:=aRect.Right;
   FRect.Left:=0;
   FRect.Top:=(FRect.Bottom div 2)-(aRect.Bottom div 2);
   FRect.Bottom:=(FRect.Bottom div 2)+(aRect.Bottom div 2)+(aRect.Bottom mod 2);
   case FChecked of
    false:Canvas.CopyRect(FRect,FHighlightGliph.Canvas,aRect);
    true:Canvas.CopyRect(FRect,FHighlightCheckGliph.Canvas,aRect);
   end;
  end;
 end
 else
 begin
  with fbmp do
  begin
   case FChecked of
    false:aRect:=Rect(0,0,FGliph.Width,FGliph.Height);
    true:aRect:=Rect(0,0,FCheckGliph.Width,FCheckGliph.Height);
   end;
   FRect:=ClientRect;
   FRect.Right:=aRect.Right;
   FRect.Left:=0;
   FRect.Top:=(FRect.Bottom div 2)-(aRect.Bottom div 2);
   FRect.Bottom:=(FRect.Bottom div 2)+(aRect.Bottom div 2)+(aRect.Bottom mod 2);
   case FChecked of
    false:Canvas.CopyRect(FRect,FGliph.Canvas,aRect);
    true:Canvas.CopyRect(FRect,FCheckGliph.Canvas,aRect);
   end;
  end;
 end;
 if not FEnabled then
 begin
  For i:=0 to fbmp.Canvas.Width do
  For r:=0 to fbmp.Canvas.Height do
  begin
   cl:=fbmp.Canvas.Colors[i,r];
   cl.blue:=cl.Blue div 2;
   cl.Red:=cl.Red div 2;
   cl.Green:=cl.Green div 2;
   fbmp.Canvas.Colors[i,r]:=cl;
  end;
 end;
 fbmp.SaveToStream(ms);
 ms.Position:=0;
 fbmp.LoadFromStream(ms);
 ms.Position:=0;
 Canvas.Draw(0,0,fbmp);
 Canvas.Brush.Style:=bsClear;
 Canvas.Font:=FFont;
 if not FEnabled then Canvas.Font.Color:=RGBToColor(Red(ColorToRGB(Canvas.Font.Color)) shr 1,Green(ColorToRGB(Canvas.Font.Color)) shr 1,Blue(ColorToRGB(Canvas.Font.Color)) shr 1);
 FRect:=ClientRect;
 if FHighlight then
 case FChecked of
  false:FRect.Left:=FHighlightGliph.Width;
  true:FRect.Left:=FHighlightCheckGliph.Width;
 end else
 case FChecked of
  false:FRect.Left:=FGliph.Width;
  true:FRect.Left:=FCheckGliph.Width;
 end;
 FRect.Left:=FRect.Left+5;
 Canvas.TextRect(FRect,FRect.Left,Height div 2 - canvas.TextHeight(FCaption) div 2,FCaption);
 fbmp.free;
 ms.free;
end;

Procedure TMaskImgRadioButton.SetFont(AFont:TFont);
begin
 FFont.Assign(AFont);
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetColor(Sender:TColor);
begin
 FColor:=sender;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetCaption(Sender:TCaption);
begin
 FCaption:=sender;
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetGliph(Sender: TBitmap);
begin
 FGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetCheckGliph(Sender: TBitmap);
begin
 FCheckGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetHighlightGliph(Sender: TBitmap);
begin
 FHighlightGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetHighlightCheckGliph(Sender: TBitmap);
begin
 FHighlightCheckGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetBackGround(Sender: TBitmap);
begin
 FBackGround.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.SetChecked(Value:boolean);
var
 i:integer;
begin
 if value then
  for i:=0 to Parent.ControlCount-1 do
   if Parent.Controls[i] is TMaskImgRadioButton then
    if (Parent.Controls[i] as TMaskImgRadioButton).GroupIndex=FGroupIndex then
     (Parent.Controls[i] as TMaskImgRadioButton).Checked:=false;
 FChecked:=value;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImgRadioButton.DoAutoSize;
var
  ModifyWidth,
  ModifyHeight : Boolean;
  NewWidth: Integer;
  NewHeight: Integer;
  FRect:TRect;
begin
  If AutoSize and (not AutoSizing) then begin
//   AutoSizing := True;
    try
      ModifyWidth := Align in [alLeft,alRight,alNone];
      ModifyHeight := Align in [alTop,alBottom,alNone];
      NewWidth:=Width;
      NewHeight:=Height;
      Canvas.Font:=FFont;
      FRect:=ClientRect;
      if FHighlight then
      case FChecked of
       false:FRect.Left:=FHighlightGliph.Width;
       true:FRect.Left:=FHighlightCheckGliph.Width;
      end else
      case FChecked of
       false:FRect.Left:=FGliph.Width;
       true:FRect.Left:=FCheckGliph.Width;
      end;
      FRect.Left:=FRect.Left+5;
      If ModifyWidth and (FGliph.Width > 0) then
        NewWidth := FRect.Left+Canvas.TextWidth(FCaption)+2;
      If ModifyHeight and (FGliph.Height > 0) then;
        NewHeight := Canvas.TextHeight(FCaption)+2;
      if (NewWidth<>Width) or (NewHeight<>Height) then begin
        SetBounds(Left,Top,NewWidth,NewHeight);
        repaint;
      end;
    finally
//      AutoSizing := False;
    end;
  end;
end;

Procedure TMaskImgRadioButton.SetEnabled(Value:boolean);
begin
 FEnabled:=Value;
 if Canvas.Locked then exit;
 Repaint;
end;

Procedure TMaskImgRadioButton.Click;
begin
 inherited Click;
 if FEnabled then Checked:=not FChecked;
end;

{TMaskImageScrollBar}

Constructor TMaskImageScrollBar.Create(AOwner:TComponent);
begin
 Inherited Create(AOwner);
 FTimer:=TTimer.Create(self);
 FTimer.Interval:=24;
 FTimer.Enabled:=false;
 FTimer.OnTimer:=@TimerEvent;
 FLastUpdate:=now;
 FCanUpdate:=true;
 FColor:=clWhite;
 FSliderColor:=clBlack;
 FCaption:=Name;
 FBackGround:=TBitmap.Create;
 FSliderBackGround:=TBitmap.Create;
 FPicture:=TBitmap.Create;
 FGliph:=TBitmap.Create;
 FDownGliph:=TBitmap.Create;
 FHighlightGliph:=TBitmap.Create;
 FEnabled:=true;
 FDown:=false;
 FHighlight:=false;
 FFont:=TFont.Create;
 FFont.OnChange:=@ImageMarignsChange;
 FImageMarigns:=TMarigns.Create;
 FImageMarigns.OnChange:=@ImageMarignsChange;
 FSliderImageMarigns:=TMarigns.Create;
 FSliderImageMarigns.OnChange:=@ImageMarignsChange;
 FSliderMarigns:=TMarigns.Create;
 FSliderMarigns.OnChange:=@ImageMarignsChange;
 FPosition:=0;
 FCount:=100;
 FMinSliderWidth:=15;
 Height:=23;
 Width:=75;
 {$IFDEF WINDOWS}
 DoubleBuffered:=true;
 {$ENDIF}
end;

Destructor TMaskImageScrollBar.Destroy;
begin
 FTimer.Free;
 FBackGround.Free;
 FSliderBackGround.Free;
 FPicture.Free;
 FGliph.Free;
 FDownGliph.Free;
 FHighlightGliph.Free;
 Inherited Destroy;
end;

procedure TMaskImageScrollBar.Assign(Source: TPersistent);
begin
  //inherited Assign(Source);
  FPosition:=(Source as TMaskImageScrollBar).FPosition;
  FCount:=(Source as TMaskImageScrollBar).FCount;
  FSliderBackGround.Assign((Source as TMaskImageScrollBar).FSliderBackGround);
  FBackGround.Assign((Source as TMaskImageScrollBar).FBackGround);
  FGliph.Assign((Source as TMaskImageScrollBar).FGliph);
  FPicture.Assign((Source as TMaskImageScrollBar).FPicture);
  FDownGliph.Assign((Source as TMaskImageScrollBar).FDownGliph);
  FHighlightGliph.Assign((Source as TMaskImageScrollBar).FHighlightGliph);
  FColor:=(Source as TMaskImageScrollBar).FColor;
  FSliderColor:=(Source as TMaskImageScrollBar).FSliderColor;
  FEnabled:=(Source as TMaskImageScrollBar).FEnabled;
  FSliderMarigns.Assign((Source as TMaskImageScrollBar).FSliderMarigns);
  FImageMarigns.Assign((Source as TMaskImageScrollBar).FImageMarigns);
  FSliderImageMarigns.Assign((Source as TMaskImageScrollBar).FSliderImageMarigns);
  FMinSliderWidth:=(Source as TMaskImageScrollBar).FMinSliderWidth;
  FKind:=(Source as TMaskImageScrollBar).FKind;
end;

Procedure TMaskImageScrollBar.ImageMarignsChange(Sender:TObject);
begin
 repaint;
end;

Procedure TMaskImageScrollBar.TimerEvent(Sender:TObject);
var
 FSliderRect:TRect;
 mp:TPoint;
begin
 Application.ProcessMessages;
 mp:=ScreenToClient(Mouse.CursorPos);
 if FDown then
 begin
  FSliderRect:=GetSliderRect;
  if FEnabled and ((IsPointInRect(mp,Rect(0,0,Width,FSliderRect.Top)) and (FKind=sbVertical)) or (IsPointInRect(mp,Rect(0,0,FSliderRect.Left,Height)) and (FKind=sbHorizontal))) then
  begin
   FPosition:=FPosition-10;
   if OnScroll<>nil then OnScroll(Self);
   Repaint;
  end else
  if FEnabled and ((IsPointInRect(mp,Rect(0,FSliderRect.Bottom,Width,Height)) and (FKind=sbVertical)) or (IsPointInRect(mp,Rect(FSliderRect.Right,0,Width,Height)) and (FKind=sbHorizontal)))  then
  begin
   FPosition:=FPosition+10;
   if OnScroll<>nil then OnScroll(Self);
   Repaint;
  end;
 end;
end;

procedure TMaskImageScrollBar.MouseLeave;
begin
 FHighlight:=false;
 inherited MouseLeave;
end;

procedure TMaskImageScrollBar.MouseEnter;
begin
 FHighlight:=true;
 inherited MouseEnter;
end;

procedure TMaskImageScrollBar.MouseMove(Shift: TShiftState; X, Y: Integer);
var
 FSliderWidth:Integer;
 FLastPos:Double;
 FSliderRect:TRect;
begin
 if not (csDesigning in ComponentState) then
 if FDown then
 begin
  FSliderRect:=GetSliderRect;
  if FEnabled and IsPointInRect(Point(X,Y),FSliderRect) then
  begin
   if FKind=sbHorizontal then
   begin
    fSliderWidth:=FSliderRect.Right-FSliderRect.Left;
    FPosition:=round((FCount-1)*(X-FMouseDownPos.X-FSliderMarigns.Left)/(Width-FSliderWidth-FSliderMarigns.Left-FSliderMarigns.Right));
   end else
   begin
    fSliderWidth:=FSliderRect.Bottom-FSliderRect.Top;
    FPosition:=round((FCount-1)*(Y-FMouseDownPos.Y-FSliderMarigns.Top)/(Height-FSliderWidth-FSliderMarigns.Bottom-FSliderMarigns.Top));
   end;
   if FPosition>FCount-1 then FPosition:=FCount-1;
   if FPosition<0 then FPosition:=0;
   FCanUpdate:=false;
   if MilliSecondsBetween(FLastUpdate,now)>82 then FCanUpdate:=true;
   if FCanUpdate and (FPosition<>FLastPos) then
   begin
    FLastPos:=FPosition;
    FLastUpdate:=now;
    if OnScroll<>nil then OnScroll(Self);
    Repaint;
   end;
  end;
 end;
 inherited MouseMove(Shift, X,Y);
end;

Procedure TMaskImageScrollBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 FSliderRect:TRect;
begin
 FSliderRect:=GetSliderRect;
 if FEnabled and IsPointInRect(Point(X,Y),FSliderRect) then
 begin
  FMouseDownPos:=Point(X-FSliderRect.Left,Y-FSliderRect.Top);
  FDown:=True;
 end else
 begin
  FDown:=True;
  FTimer.Enabled:=true;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 Repaint;
end;

Procedure TMaskImageScrollBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 fflag:boolean;
 i:integer;
begin
 if FEnabled then
 begin
  FTimer.Enabled:=False;
  FDown:=False;
 end;
 Inherited MouseDown(Button,Shift,X,Y);
 Repaint;
end;

Function TMaskImageScrollBar.GetSliderRect:TRect;
var
 fSliderWidth:integer;
 fSliderRect:TRect;
begin
 if FKind=sbHorizontal then
 begin
  fSliderWidth:=round((Width-FSliderMarigns.Left-FSliderMarigns.Right)*((10+ln(1/FCount))/10));
  if FSliderWidth<FMinSliderWidth then
   fSliderWidth:=FMinSliderWidth;
  fSliderRect.Top:=FSliderMarigns.Top;
  fSliderRect.Bottom:=ClientRect.Bottom-FSliderMarigns.Bottom;
  fSliderRect.Left:=round(FSliderMarigns.Left+(Width-FSliderMarigns.Left-FSliderMarigns.Right-FSliderWidth)*FPosition/(FCount-1));
  fSliderRect.Right:=fSliderRect.Left+FSliderWidth;
 end else
 begin
  fSliderWidth:=round((Height-FSliderMarigns.Top-FSliderMarigns.Bottom)*((10+ln(1/FCount))/10));
  if FSliderWidth<FMinSliderWidth then
   fSliderWidth:=FMinSliderWidth;
  fSliderRect.Left:=FSliderMarigns.Left;
  fSliderRect.Right:=ClientRect.Right-FSliderMarigns.Right;
  fSliderRect.Top:=round(FSliderMarigns.Top+(Height-FSliderMarigns.Top-FSliderMarigns.Bottom-FSliderWidth)*FPosition/(FCount-1));
  fSliderRect.Bottom:=fSliderRect.Top+FSliderWidth;
 end;
 result:=fSliderRect;
end;

Procedure TMaskImageScrollBar.Paint;
var
 fSliderRect:TRect;
 aoffset:integer;
 fbmp,ftmp:TBitmap;
begin
 fbmp:=TBitmap.Create;
 fbmp.Width:=width;
 fbmp.Height:=Height;
 if FColor=clNone then
 begin
  fbmp.Canvas.Brush.Style:=bsClear;
  fbmp.Transparent:=true;
 end else fbmp.Canvas.Brush.Style:=bsSolid;
 fbmp.Canvas.Brush.Color:=FColor;
 fbmp.Canvas.FillRect(ClientRect);
 if IsImageAssigned(FBackGround) then
 begin
  fbmp.Canvas.CopyRect(ClientRect,FBackGround.Canvas,FBackGround.Canvas.ClipRect);
 end;
 if IsImageAssigned(FSliderBackGround) then
 begin
  DrawImageWithBorder(fbmp,FSliderBackGround,ClientRect,FImageMarigns);
 end;
 FSliderRect:=GetSliderRect;
 //draw slider
 FTmp:=nil;
 if FDown then
 begin
  if IsImageAssigned(FDownGliph) then
   FTmp:=FDownGliph;
 end
 else if FHighlight then
 begin
  if IsImageAssigned(FHighlightGliph) then
   FTmp:=FHighlightGliph;
 end
 else
 begin
  if IsImageAssigned(FGliph) then
   FTmp:=FGliph;
 end;
 if FSliderColor=clNone then
 begin
  fbmp.Canvas.Brush.Style:=bsClear;
 end else fbmp.Canvas.Brush.Style:=bsSolid;
 fbmp.Canvas.Brush.Color:=FSliderColor;
 fbmp.Canvas.FillRect(FSliderRect);
 if ftmp<>nil then
  DrawImageWithBorder(fbmp,ftmp,fSliderRect,FSliderImageMarigns);
 fbmp.Mask(clDefault);
 Canvas.Draw(0,0,fbmp);
 Canvas.Brush.Style:=bsClear;
 Canvas.Font:=FFont;
 aoffset:=0;
 if (FPicture.Width<>0) and (FCaption<>'') then aoffset:=2;
 if not FEnabled then Canvas.Font.Color:=RGBToColor(Red(ColorToRGB(Canvas.Font.Color)) shr 1,Green(ColorToRGB(Canvas.Font.Color)) shr 1,Blue(ColorToRGB(Canvas.Font.Color)) shr 1);
 if FDown then
 begin
  Canvas.Draw(width div 2 - canvas.TextWidth(FCaption) div 2+1-aoffset div 2-FPicture.Width div 2,Height div 2 - FPicture.Height div 2 +1,FPicture);
  Canvas.TextRect(ClientRect,aoffset div 2+FPicture.Width div 2+width div 2 - canvas.TextWidth(FCaption) div 2+1,Height div 2 - canvas.TextHeight(FCaption) div 2+1,FCaption);
 end else
 begin
  Canvas.Draw(width div 2 - canvas.TextWidth(FCaption) div 2-aoffset div 2-FPicture.Width div 2,Height div 2 - FPicture.Height div 2,FPicture);
  Canvas.TextRect(ClientRect,aoffset div 2+FPicture.Width div 2+width div 2 - canvas.TextWidth(FCaption) div 2,Height div 2 - canvas.TextHeight(FCaption) div 2,FCaption);
 end;
 fbmp.free;
end;

Procedure TMaskImageScrollBar.SetImageMarigns(Value:TMarigns);
begin
 if FImageMarigns=Value then exit;
 FImageMarigns.Assign(Value);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetSliderImageMarigns(Value:TMarigns);
begin
 if FSliderImageMarigns=Value then exit;
 FSliderImageMarigns.Assign(Value);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetSliderMarigns(Value:TMarigns);
begin
 if FSliderMarigns=Value then exit;
 FSliderMarigns.Assign(Value);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TmaskImageScrollBar.SetKind(Value:TScrollBarKind);
begin
 FKind:=Value;
 repaint;
end;

procedure TMaskImageScrollBar.SetEnabled(const AValue: boolean);
begin
  FEnabled:=AValue;
end;

procedure TMaskImageScrollBar.SetCaption(const AValue: TCaption);
begin
 FCaption:=AValue;
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

procedure TMaskImageScrollBar.SetFont(const AValue: TFont);
begin
 FFont.Assign(AValue);
 if AutoSize then
 begin
  AutoSize:=false;
  AutoSize:=true;
 end;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetColor(Sender:TColor);
begin
 FColor:=sender;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetSliderColor(Sender:TColor);
begin
 FSliderColor:=sender;
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetCount(Value:double);
begin
 if Value<2 then Value:=2;
 FCount:=Value;
 if FPosition>FCount-1 then FPosition:=FCount-1;
 repaint;
end;

Procedure TMaskImageScrollBar.SetMinSliderWidth(Value:integer);
begin
 FMinSliderWidth:=Value;
 repaint;
end;

Procedure TMaskImageScrollBar.SetPosition(Value:double);
begin
 FPosition:=Value;
 if FPosition<0 then FPosition:=0;
 if FPosition>FCount-1 then FPosition:=FCount-1;
 repaint;
end;

Procedure TMaskImageScrollBar.SetGliph(Sender: TBitmap);
begin
 FGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetDownGliph(Sender: TBitmap);
begin
 FDownGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetHighlightGliph(Sender: TBitmap);
begin
 FHighlightGliph.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetBackGround(Sender: TBitmap);
begin
 FBackGround.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

Procedure TMaskImageScrollBar.SetSliderBackGround(Sender: TBitmap);
begin
 FSliderBackGround.Assign(Sender);
 if Canvas.Locked then exit;
 repaint;
end;

procedure Register;
begin
 RegisterComponents('MaskImage Controls', [TMaskImageBtn,TMaskImageScrollBar,TMaskImgCheckBox,TMaskImgRadioButton]);
end;

end.
