unit EditEx;

{$mode objfpc}{$H+}

interface

uses
  {$IFDEF WIN32}Windows, {$ENDIF}Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls,
  LMessages, ExtCtrls, ComCtrls, MaskImageBtn, ImgList, PropEdits;

type

  TBorderDrawEvent = procedure(Sender: TObject; aCanvas: TCanvas; var DefaultDraw:boolean) of object;

  { TMyTreeView }

  TMyTreeView=class(TTreeView)
  private
    FOnCustomSelection:TNotifyEvent;
  protected
    procedure DoSelectionChanged; override;
  public
    constructor Create(AnOwner: TComponent); override;
    procedure Assign(Source: TPersistent); override;
    property ScrolledLeft;
    property ScrolledTop;
    property MaxScrollTop: integer read GetMaxScrollTop;
    property OnCustomSelection:TNotifyEvent read FOnCustomSelection write FOnCustomSelection;
  end;

  { TEditEx }

  TEditEx = class(TEdit)
  private
    FHover:boolean;
    FDraw:TBorderDrawEvent;
    FBorderWidth:integer;
    FBorderColor:TColor;
    FBorderLightColor:TColor;
    FBorderSelectedColor:TColor;
    FBorderSelectedLightColor:TColor;
  protected
    procedure WMPaint(var Msg: TLMPaint); message LM_PAINT;
    procedure DoDraw(aCanvas:TCanvas);
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Assign(Source: TPersistent); override;
  published
    property BorderWidth:integer read FBorderWidth write FBorderWidth;
    property BorderColor:TColor read FBorderColor write FBorderColor;
    property BorderLightColor:TColor read FBorderLightColor write FBorderLightColor;
    property BorderSelectedColor:TColor read FBorderSelectedColor write FBorderSelectedColor;
    property BorderSelectedLightColor:TColor read FBorderSelectedLightColor write FBorderSelectedLightColor;
    property OnDraw: TBorderDrawEvent read FDraw write FDraw;
    property OnMouseLeave;
    property OnMouseEnter;
    Property Align;
    Property Hint;
    Property ShowHint;
    Property ParentShowHint;
    Property Visible;
    Property Anchors;
  end;

  { TTreeList }

  TTreeList = class(TCustomControl)
  private
    FTreeView:TMyTreeView;
    FHover:boolean;
    FBorderWidth:integer;
    FBorderColor:TColor;
    FBorderLightColor:TColor;
    FBorderSelectedColor:TColor;
    FBorderSelectedLightColor:TColor;
    FScrollBar:TMaskImageScrollBar;
    FTimer:TTimer;
    FCanDrawArrow:boolean;
    FNode:TTreeNode;
    FExpanding:boolean;
    FCanSelect,Deactivated:boolean;
    FSelectedFont:TFont;
    FArrowSize:integer;
    FArrowColor:TColor;
    FAllowRootArrow:boolean;
    FOnUpdateScroll:TNotifyEvent;
    procedure DrawArrow(Sender: TCustomTreeView; Node: TTreeNode; Angle: double);
    function GetBackGround: TBitmap;
    function GetDownGliph: TBitmap;
    function GetGliph: TBitmap;
    function GetHighlightGliph: TBitmap;
    function GetImageMarigns: TMarigns;
    function GetImages: TCustomImageList;
    function GetItems: TTreeNodes;
    function GetMinSliderWidth: integer;
    function GetSBColor: TColor;
    function GetSelection: TTreeNode;
    function GetSelectionChanged: TNotifyEvent;
    function GetSliderBackGround: TBitmap;
    function GetSliderColor: TColor;
    function GetSliderImageMarigns: TMarigns;
    function GetSliderMarigns: TMarigns;
    function GetTopItem: TTreeNode;
    procedure SetAllowRootArrow(const AValue: boolean);
    procedure SetBackGround(const AValue: TBitmap);
    procedure SetDownGliph(const AValue: TBitmap);
    procedure SetFont(const AValue: TFont);
    procedure SetGliph(const AValue: TBitmap);
    procedure SetHighlightGliph(const AValue: TBitmap);
    procedure SetImageMarigns(const AValue: TMarigns);
    procedure SetImages(const AValue: TCustomImageList);
    procedure SetItems(const AValue: TTreeNodes);
    procedure SetMinSliderWidth(const AValue: integer);
    procedure SetSBColor(const AValue: TColor);
    procedure SetSelectedFont(const AValue: TFont);
    procedure SetSelection(const AValue: TTreeNode);
    procedure SetSelectionChanged(const AValue: TNotifyEvent);
    procedure SetScrollBarWidth(Value:integer);
    function  GetScrollBarWidth:integer;
    function  GetFont:TFont;
    function  GetColor:TColor;
    procedure SetColor(Value:TColor);
    procedure SetSliderColor(const AValue: TColor);
    procedure SetSliderImageMarigns(const AValue: TMarigns);
    procedure SetSliderMarigns(const AValue: TMarigns);
    procedure SetTopItem(const AValue: TTreeNode);
    procedure TimerTimer(Sender: TObject);
    procedure TreeListCollapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure TreeListCollapsed(Sender: TObject; Node: TTreeNode);
    procedure TreeListExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TreeListExpanded(Sender: TObject; Node: TTreeNode);
    procedure TreeListCustomDrawItem(Sender: TCustomTreeView; Node: TTreeNode;
      State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure TreeListCustomDraw(Sender: TCustomTreeView; const ARect: TRect;
      var DefaultDraw: Boolean);
    procedure ScrollBarScroll(Sender: TObject);
    procedure TreeListEnter(Sender: TObject);
    procedure TreeListExit(Sender: TObject);
    procedure TreeListMouseEnter(Sender: TObject);
    procedure TreeListMouseLeave(Sender: TObject);
    procedure TreeListWhell(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure TreeListCustomSelection(Sender: TObject);
  protected
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure DoOnResize; override;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
    procedure UpdateScroll; virtual;
    property TreeView:TMyTreeView read FTreeView;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;
    procedure Assign(Source: TPersistent); override;
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure FullCollapse;
    procedure FullExpand;
  published
    property Selected: TTreeNode read GetSelection write SetSelection;
    property TopItem: TTreeNode read GetTopItem write SetTopItem;
    property Images: TCustomImageList read GetImages write SetImages;
    property Items: TTreeNodes read GetItems write SetItems;
    property SliderGliph:TBitmap read GetGliph Write SetGliph;
    property SliderDownGliph:TBitmap read GetDownGliph Write SetDownGliph;
    property SliderHighlightGliph:TBitmap read GetHighlightGliph Write SetHighlightGliph;
    property BackGround:TBitmap read GetBackGround Write SetBackGround;
    property SliderBackGround:TBitmap read GetSliderBackGround Write SetBackGround;
    property SliderColor:TColor read GetSliderColor write SetSliderColor default clBlack;
    property ScrollBarColor:TColor read GetSBColor write SetSBColor default clWhite;
    property ImageMarigns:TMarigns read GetImageMarigns write SetImageMarigns;
    property SliderImageMarigns:TMarigns read GetSliderImageMarigns write SetSliderImageMarigns;
    property SliderMarigns:TMarigns read GetSliderMarigns write SetSliderMarigns;
    property MinSliderWidth:integer read GetMinSliderWidth write SetMinSliderWidth;
    property BorderWidth:integer read FBorderWidth write FBorderWidth;
    property BorderColor:TColor read FBorderColor write FBorderColor;
    property BorderLightColor:TColor read FBorderLightColor write FBorderLightColor;
    property BorderSelectedColor:TColor read FBorderSelectedColor write FBorderSelectedColor;
    property BorderSelectedLightColor:TColor read FBorderSelectedLightColor write FBorderSelectedLightColor;
    property ScrollBarWidth:integer read GetScrollBarWidth write SetScrollBarWidth;
    property ArrowSize:integer read FArrowSize write FArrowSize;
    property ArrowColor:TColor read FArrowColor write FArrowColor;
    property Font:TFont read GetFont write SetFont;
    property SelectedFont:TFont read FSelectedFont write SetSelectedFont;
    property Color:TColor read GetColor write SetColor;
    property AllowRootArrow:boolean read FAllowRootArrow write SetAllowRootArrow;
    Property Align;
    Property Hint;
    Property ShowHint;
    Property ParentShowHint;
    Property Visible;
    Property Anchors;
    property OnUpdateScroll:TNotifyEvent read FOnUpdateScroll write FOnUpdateScroll;
    property OnSelectionChanged: TNotifyEvent
      read GetSelectionChanged write SetSelectionChanged;
    property OnEnter;
    property OnExit;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
  end;

  { TListBoxEx }

  TListBoxEx = class(TListBox)
  private
    FHover:boolean;
    FDraw:TBorderDrawEvent;
    FBorderWidth:integer;
    FBorderColor:TColor;
    FBorderLightColor:TColor;
    FBorderSelectedColor:TColor;
    FBorderSelectedLightColor:TColor;
  protected
    procedure WMPaint(var Msg: TLMPaint); message LM_PAINT;
    procedure DoDraw(aCanvas:TCanvas);
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure MouseEnter; override;
    procedure MouseLeave; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Assign(Source: TPersistent); override;
  published
    property BorderWidth:integer read FBorderWidth write FBorderWidth;
    property BorderColor:TColor read FBorderColor write FBorderColor;
    property BorderLightColor:TColor read FBorderLightColor write FBorderLightColor;
    property BorderSelectedColor:TColor read FBorderSelectedColor write FBorderSelectedColor;
    property BorderSelectedLightColor:TColor read FBorderSelectedLightColor write FBorderSelectedLightColor;
    Property Align;
    Property Hint;
    Property ShowHint;
    Property ParentShowHint;
    Property Visible;
    Property Anchors;
    property OnDraw: TBorderDrawEvent read FDraw write FDraw;
    property OnMouseLeave;
    property OnMouseEnter;
  end;

  { TComboBoxEx }

  TComboBoxEx = class(TCustomControl)
  private
    FEdit:TEditEx;
    FButton:TMaskImageBtn;
    FTreeListForm:TForm;
    FSelectionChanged:TNotifyEvent;
    FEnabled:boolean;
    FLeftSibling,FRightSibling:TControl;
    FOnUpdateScroll:TNotifyEvent;
    function GetAllowRootArrow: boolean;
    function GetArrowColor: TColor;
    function GetArrowSize: integer;
    function GetBackGround: TBitmap;
    function GetBorderColor: TColor;
    function GetBorderLightColor: TColor;
    function GetBorderSelectedColor: TColor;
    function GetBorderSelectedLightColor: TColor;
    function GetBorderWidth: integer;
    function GetButtonColor: TColor;
    function GetButtonFont: TFont;
    function GetCaption: TCaption;
    function GetColor: TColor;
    function GetComboHeight: integer;
    function GetComboListBorderColor: TColor;
    function GetComboListBorderLightColor: TColor;
    function GetComboListBorderSelectedColor: TColor;
    function GetComboListBorderSelectedLightColor: TColor;
    function GetComboListBorderWidth: integer;
    function GetComboListColor: TColor;
    function GetComboListFont: TFont;
    function GetDownGliph: TBitmap;
    function GetDownHighlightGliph: TBitmap;
    function GetFont: TFont;
    function GetGliph: TBitmap;
    function GetHighlightGliph: TBitmap;
    function GetImageMarigns: TMarigns;
    function GetImages: TCustomImageList;
    function GetItems: TTreeNodes;
    function GetMinSliderWidth: integer;
    function GetPicture: TBitmap;
    function GetSBColor: TColor;
    function GetScrollBarWidth: integer;
    function GetSelectedFont: TFont;
    function GetSelection: TTreeNode;
    function GetSliderBackGround: TBitmap;
    function GetSliderColor: TColor;
    function GetSliderImageMarigns: TMarigns;
    function GetSliderMarigns: TMarigns;
    function GetText: TCaption;
    function GetTreeList: TTreeList;
    procedure SetAllowRootArrow(const AValue: boolean);
    procedure SetArrowColor(const AValue: TColor);
    procedure SetArrowSize(const AValue: integer);
    procedure SetBackGround(const AValue: TBitmap);
    procedure SetBorderColor(const AValue: TColor);
    procedure SetBorderLightColor(const AValue: TColor);
    procedure SetBorderSelectedColor(const AValue: TColor);
    procedure SetBorderSelectedLightColor(const AValue: TColor);
    procedure SetBorderWidth(const AValue: integer);
    procedure SetButtonColor(const AValue: TColor);
    procedure SetButtonFont(const AValue: TFont);
    procedure SetCaption(const AValue: TCaption);
    procedure SetColor(const AValue: TColor);
    procedure SetComboHeight(const AValue: integer);
    procedure SetComboListBorderColor(const AValue: TColor);
    procedure SetComboListBorderLightColor(const AValue: TColor);
    procedure SetComboListBorderSelectedColor(const AValue: TColor);
    procedure SetComboListBorderSelectedLightColor(const AValue: TColor);
    procedure SetComboListBorderWidth(const AValue: integer);
    procedure SetComboListColor(const AValue: TColor);
    procedure SetComboListFont(const AValue: TFont);
    procedure SetDownGliph(const AValue: TBitmap);
    procedure SetDownHighlightGliph(const AValue: TBitmap);
    procedure SetEnabled(const AValue: boolean);
    procedure SetFont(const AValue: TFont);
    procedure SetGliph(const AValue: TBitmap);
    procedure SetHighlightGliph(const AValue: TBitmap);
    procedure SetImageMarigns(const AValue: TMarigns);
    procedure SetImages(const AValue: TCustomImageList);
    procedure SetItems(const AValue: TTreeNodes);
    procedure SetMinSliderWidth(const AValue: integer);
    procedure SetPicture(const AValue: TBitmap);
    procedure SetSBColor(const AValue: TColor);
    procedure SetScrollBarWidth(const AValue: integer);
    procedure SetSelectedFont(const AValue: TFont);
    procedure SetSelection(const AValue: TTreeNode);
    procedure SetSliderColor(const AValue: TColor);
    procedure SetSliderImageMarigns(const AValue: TMarigns);
    procedure SetSliderMarigns(const AValue: TMarigns);
    procedure SetText(const AValue: TCaption);
  protected
    procedure DoOnResize; override;
    procedure TreeSelectionUpdate(Sender: TObject);
    procedure TreeSelectionChanged(Sender: TObject);
    procedure ComboDownList(Sender: TObject);
    property TreeList:TTreeList read GetTreeList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;
    procedure Assign(Source: TPersistent); override;
    property ComboHeight:integer read GetComboHeight write SetComboHeight;
  published
    property BorderWidth:integer read GetBorderWidth write SetBorderWidth;
    property BorderColor:TColor read GetBorderColor write SetBorderColor;
    property BorderLightColor:TColor read GetBorderLightColor write SetBorderLightColor;
    property BorderSelectedColor:TColor read GetBorderSelectedColor write SetBorderSelectedColor;
    property BorderSelectedLightColor:TColor read GetBorderSelectedLightColor write SetBorderSelectedLightColor;
    property Text:TCaption read GetText write SetText;
    property Color:TColor read GetColor write SetColor;
    property Font:TFont read GetFont write SetFont;
    {Button}
    Property Gliph:TBitmap read GetGliph Write SetGliph;
    Property ArrowImage:TBitmap read GetPicture Write SetPicture;
    Property DownGliph:TBitmap read GetDownGliph Write SetDownGliph;
    Property DownHighlightGliph:TBitmap read GetDownHighlightGliph Write SetDownHighlightGliph;
    Property HighlightGliph:TBitmap read GetHighlightGliph Write SetHighlightGliph;
    Property BackGround:TBitmap read GetBackGround Write SetBackGround;
    Property ButtonColor:TColor read GetButtonColor write SetButtonColor default clBtnFace;
    Property ButtonCaption:TCaption read GetCaption write SetCaption;
    Property Enabled:boolean read FEnabled write SetEnabled default true;
    Property ButtonFont:TFont read GetButtonFont write SetButtonFont;
    Property ButtonMarigns:TMarigns read GetImageMarigns write SetImageMarigns;
    {TreeList}
    property Selected: TTreeNode read GetSelection write SetSelection;
    property Images: TCustomImageList read GetImages write SetImages;
    property Items: TTreeNodes read GetItems write SetItems;
    property SliderGliph:TBitmap read GetGliph Write SetGliph;
    property SliderDownGliph:TBitmap read GetDownGliph Write SetDownGliph;
    property SliderHighlightGliph:TBitmap read GetHighlightGliph Write SetHighlightGliph;
    property ScrollBarBackGround:TBitmap read GetBackGround Write SetBackGround;
    property SliderBackGround:TBitmap read GetSliderBackGround Write SetBackGround;
    property SliderColor:TColor read GetSliderColor write SetSliderColor default clBlack;
    property ScrollBarColor:TColor read GetSBColor write SetSBColor default clWhite;
    property ScrollBarImageMarigns:TMarigns read GetImageMarigns write SetImageMarigns;
    property SliderImageMarigns:TMarigns read GetSliderImageMarigns write SetSliderImageMarigns;
    property SliderMarigns:TMarigns read GetSliderMarigns write SetSliderMarigns;
    property MinSliderWidth:integer read GetMinSliderWidth write SetMinSliderWidth;
    property ComboListBorderWidth:integer read GetComboListBorderWidth write SetComboListBorderWidth;
    property ComboListBorderColor:TColor read GetComboListBorderColor write SetComboListBorderColor;
    property ComboListBorderLightColor:TColor read GetComboListBorderLightColor write SetComboListBorderLightColor;
    property ComboListBorderSelectedColor:TColor read GetComboListBorderSelectedColor write SetComboListBorderSelectedColor;
    property ComboListBorderSelectedLightColor:TColor read GetComboListBorderSelectedLightColor write SetComboListBorderSelectedLightColor;
    property ScrollBarWidth:integer read GetScrollBarWidth write SetScrollBarWidth;
    property ComboListArrowSize:integer read GetArrowSize write SetArrowSize;
    property ComboListArrowColor:TColor read GetArrowColor write SetArrowColor;
    property ComboListFont:TFont read GetComboListFont write SetComboListFont;
    property ComboListSelectedFont:TFont read GetSelectedFont write SetSelectedFont;
    property ComboListColor:TColor read GetComboListColor write SetComboListColor;
    property LeftSibling:TControl read FLeftSibling write FLeftSibling;
    property RightSibling:TControl read FRightSibling write FRightSibling;
    property AllowRootArrow:boolean read GetAllowRootArrow write SetAllowRootArrow;
    {ComboBoxEx}
    Property Align;
    Property Hint;
    Property ShowHint;
    Property ParentShowHint;
    Property Visible;
    Property Anchors;
    property OnUpdateScroll:TNotifyEvent read FOnUpdateScroll write FOnUpdateScroll;
    property OnSelectionChanged: TNotifyEvent read FSelectionChanged write FSelectionChanged;
    property OnEnter;
    property OnExit;
    property OnMouseMove;
    property OnMouseLeave;
    property OnMouseEnter;
  end;

//procedure to set forms transparency (external GTK+)
{$IFDEF LINUX}procedure gdk_window_set_opacity(window:PGdkWindow; opacity:gdouble); cdecl; external gdklib;{$ENDIF}
Procedure DecRect(var aRect:TRect; aValue:integer=1);

procedure Register;

implementation

uses combotreelistform;

//Procedure that change rectangle size by dec borders value around
Procedure DecRect(var aRect:TRect; aValue:integer=1);
begin
  Inc(aRect.Left,aValue);
  Dec(aRect.Right,aValue);
  Inc(aRect.Top,aValue);
  Dec(aRect.Bottom,aValue);
end;

Function IntToStr(Value:integer;Digits:integer=0):string;
var
  s:string;
begin
  s:=SysUtils.IntToStr(Value);
  while length(s)<digits do
    s:='0'+s;
  result:=s;
end;

procedure TEditEx.WMPaint(var Msg: TLMPaint);
var
  aCanvas:TCanvas; //control canvas
  aRect:TRect;//drawing rectangle
  aDefaultDraw:boolean;
begin
  Inherited WMPaint(msg);
  aDefaultDraw:=true;
  aCanvas:=TControlCanvas.Create; //get control canvas
  (aCanvas as TControlCanvas).Control:=self; //set drawing control
  aCanvas.Handle:={$IFDEF WIN32}GetWindowDC{$ELSE}GetDC{$ENDIF}(Handle); //and drawing context
  aRect:=BoundsRect;//drawing rect is a control rect
  aRect.Top:=aRect.Top-Top;
  aRect.Bottom:=aRect.Bottom-Top;
  aRect.Left:=aRect.Left-Left;
  aRect.Right:=aRect.Right-Left;
  aCanvas.ClipRect:=aRect;
  aCanvas.Lock; //lock drawing to canvas
  if FDraw<>nil then FDraw(self,aCanvas,aDefaultDraw);
  if aDefaultDraw then DoDraw(aCanvas);
  aCanvas.Unlock; //paint new canvas
  ReleaseDC(Handle,aCanvas.Handle); //release drawing context
  aCanvas.Free; //free memory from drawing surface class
end;

procedure TEditEx.DoDraw(aCanvas:TCanvas);
var
  aRect:TRect;//drawing rectangle
  i:integer;
begin
  aRect:=aCanvas.ClipRect;
  aCanvas.Brush.Style:=bsClear; //set fill style as transparent
  aCanvas.Pen.Color:=Color;//set rect color to a control background color
  aCanvas.Rectangle(aRect); //draw single line rect
  DecRect(aRect); //decrease drawing rectangle size by one pixel and do same two times
  aCanvas.Rectangle(aRect);
  DecRect(aRect);
  aCanvas.Rectangle(aRect);
  DecRect(aRect,-2);
  for i:=1 to FBorderWidth do
  begin
    if FHover then
      if Focused then //if control focused
        aCanvas.Pen.Color:=FBorderSelectedLightColor //set blue border color
      else
        aCanvas.Pen.Color:=FBorderLightColor //set gray border color
    else
      if Focused then //if control focused
        aCanvas.Pen.Color:=FBorderSelectedColor //set blue border color
      else
        aCanvas.Pen.Color:=FBorderColor; //set gray border color
      aCanvas.Rectangle(aRect);
    DecRect(aRect);
  end;
end;

procedure TEditEx.DoEnter;
begin
  inherited DoEnter;
  Repaint;
end;

procedure TEditEx.DoExit;
begin
  inherited DoExit;
  Repaint;
end;

procedure TEditEx.MouseEnter;
begin
  inherited MouseEnter;
  FHover:=true;
  Repaint;
end;

procedure TEditEx.MouseLeave;
begin
  inherited MouseLeave;
  FHover:=false;
  Repaint;
end;

constructor TEditEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHover:=false;
  FBorderColor:=clBlack;
  FBorderLightColor:=clDkGray;
  FBorderSelectedColor:=clGray;
  FBorderSelectedLightColor:=clLtGray;
  FBorderWidth:=1;
end;

procedure TEditEx.Assign(Source: TPersistent);
begin
  //inherited Assign(Source);
  FBorderWidth:=(Source as TEditEx).FBorderWidth;
  FBorderColor:=(Source as TEditEx).FBorderColor;
  FBorderLightColor:=(Source as TEditEx).FBorderLightColor;
  FBorderSelectedColor:=(Source as TEditEx).FBorderSelectedColor;
  FBorderSelectedLightColor:=(Source as TEditEx).FBorderSelectedLightColor;
end;

procedure TListBoxEx.WMPaint(var Msg: TLMPaint);
var
  aCanvas:TCanvas; //control canvas
  aRect:TRect;//drawing rectangle
  aDefaultDraw:boolean;
begin
  Inherited WMPaint(msg);
  aDefaultDraw:=true;
  aCanvas:=TControlCanvas.Create; //get control canvas
  (aCanvas as TControlCanvas).Control:=self; //set drawing control
  aCanvas.Handle:={$IFDEF WIN32}GetWindowDC{$ELSE}GetDC{$ENDIF}(Handle); //and drawing context
  aRect:=BoundsRect;//drawing rect is a control rect
  aRect.Top:=aRect.Top-Top;
  aRect.Bottom:=aRect.Bottom-Top;
  aRect.Left:=aRect.Left-Left;
  aRect.Right:=aRect.Right-Left;
  aCanvas.ClipRect:=aRect;
  aCanvas.Lock; //lock drawing to canvas
  if FDraw<>nil then FDraw(self,aCanvas,aDefaultDraw);
  if aDefaultDraw then DoDraw(aCanvas);
  aCanvas.Unlock; //paint new canvas
  ReleaseDC(Handle,aCanvas.Handle); //release drawing context
  aCanvas.Free; //free memory from drawing surface class
end;

procedure TListBoxEx.DoDraw(aCanvas: TCanvas);
var
  aRect:TRect;//drawing rectangle
  i:integer;
begin
  aRect:=aCanvas.ClipRect;
  aCanvas.Brush.Style:=bsClear; //set fill style as transparent
  aCanvas.Pen.Color:=Color;//set rect color to a control background color
  aCanvas.Rectangle(aRect); //draw single line rect
  DecRect(aRect); //decrease drawing rectangle size by one pixel and do same two times
  aCanvas.Rectangle(aRect);
  DecRect(aRect);
  aCanvas.Rectangle(aRect);
  DecRect(aRect,-2);
  for i:=1 to FBorderWidth do
  begin
    if FHover then
      if Focused then //if control focused
        aCanvas.Pen.Color:=FBorderSelectedLightColor //set blue border color
      else
        aCanvas.Pen.Color:=FBorderLightColor //set gray border color
    else
      if Focused then //if control focused
        aCanvas.Pen.Color:=FBorderSelectedColor //set blue border color
      else
        aCanvas.Pen.Color:=FBorderColor; //set gray border color
      aCanvas.Rectangle(aRect);
    DecRect(aRect);
  end;
end;

procedure TListBoxEx.DoEnter;
begin
  inherited DoEnter;
  Repaint;
end;

procedure TListBoxEx.DoExit;
begin
  inherited DoExit;
  Repaint;
end;

procedure TListBoxEx.MouseEnter;
begin
  inherited MouseEnter;
  FHover:=True;
  Repaint;
end;

procedure TListBoxEx.MouseLeave;
begin
  inherited MouseLeave;
  FHover:=False;
  Repaint;
end;

constructor TListBoxEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHover:=false;
  FBorderColor:=clBlack;
  FBorderLightColor:=clDkGray;
  FBorderSelectedColor:=clGray;
  FBorderSelectedLightColor:=clLtGray;
  FBorderWidth:=1;
end;

procedure TListBoxEx.Assign(Source: TPersistent);
begin
//  inherited Assign(Source);
  FBorderWidth:=(Source as TListBoxEx).FBorderWidth;
  FBorderColor:=(Source as TListBoxEx).FBorderColor;
  FBorderLightColor:=(Source as TListBoxEx).FBorderLightColor;
  FBorderSelectedColor:=(Source as TListBoxEx).FBorderSelectedColor;
  FBorderSelectedLightColor:=(Source as TListBoxEx).FBorderSelectedLightColor;
end;

procedure Register;
begin
  RegisterComponents('MaskImage Controls',[TEditEx,TListBoxEx,TTreeList,TComboBoxEx]);
end;

{ TTreeList }

procedure TTreeList.TreeListEnter(Sender: TObject);
begin
  FTreeView.Repaint;
end;

procedure TTreeList.TreeListExit(Sender: TObject);
begin
  FTreeView.Repaint;
end;

procedure TTreeList.TreeListMouseEnter(Sender: TObject);
begin
  FHover:=true;
  FTreeView.Repaint;
end;

procedure TTreeList.TreeListMouseLeave(Sender: TObject);
begin
  FHover:=false;
  FTreeView.Repaint;
end;

procedure TTreeList.TreeListWhell(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  Handled:=true;
  FScrollBar.Position:=FScrollBar.Position-WheelDelta/3;
  FScrollBar.OnScroll(FScrollBar);
end;

procedure TTreeList.TreeListCustomSelection(Sender: TObject);
begin
  FScrollBar.Position:=FTreeView.ScrolledTop;
end;

Procedure TTreeList.DrawArrow(Sender: TCustomTreeView; Node: TTreeNode; Angle:double);

  procedure DrawArrowHead(Canvas: TCanvas; X,Y: Integer; Angle,LW: Extended);
  var
    A1,A2,w1,w2: Extended;
    Arrow: array[0..3] of TPoint;
    OldWidth: Integer;
  const
    Beta=0.322;
    LineLen=4.74;
    CentLen=3;
  begin
    Angle:=Pi+Angle;
    A1:=Angle-Beta;
    A2:=Angle+Beta;
    w1:=LineLen*LW*Cos(A1);
    if w1<CentLen*LW*Cos(Angle) then
      w1:=CentLen*LW*Cos(Angle);
    if w1<LineLen*LW*Cos(A2) then
      w1:=LineLen*LW*Cos(A2);
    w2:=LineLen*LW*Sin(A1);
    if w2<CentLen*LW*Sin(Angle) then
      w2:=CentLen*LW*Sin(Angle);
    if w2<LineLen*LW*Sin(A2) then
      w2:=LineLen*LW*Sin(A2);
    Arrow[0]:=Point(X-round(w1/2),Y+round(w2/2));
    Arrow[1]:=Point(X+Round(LineLen*LW*Cos(A1))-round(w1/2),Y-Round(LineLen*LW*Sin(A1))+round(w2/2));
    Arrow[2]:=Point(X+Round(CentLen*LW*Cos(Angle))-round(w1/2),Y-Round(CentLen*LW*Sin(Angle))+round(w2/2));
    Arrow[3]:=Point(X+Round(LineLen*LW*Cos(A2))-round(w1/2),Y-Round(LineLen*LW*Sin(A2))+round(w2/2));
    OldWidth:=Canvas.Pen.Width;
    Canvas.Brush.Style:=bsSolid;
    Canvas.Brush.Color:=FArrowColor;
    Canvas.Pen.Color:=FArrowColor;
    Canvas.Pen.Style:=psSolid;
    Canvas.Pen.Width:=1;
    Canvas.Polygon(Arrow);
    Canvas.Pen.Width:=OldWidth
  end;

var
  aRect:TRect;
begin
  aRect:=Node.DisplayRect(true);
  Sender.Canvas.Brush.Color:=clWhite;
  Sender.Canvas.Brush.Style:=bsSolid;
  Sender.Canvas.FillRect(Rect(aRect.Left-FTreeView.DefaultItemHeight,aRect.Top,aRect.Left,aRect.Bottom));
  DrawArrowHead(Sender.Canvas,aRect.Left-FTreeView.DefaultItemHeight div 2+FArrowSize+3,aRect.Top+(aRect.Bottom-aRect.Top) div 2-1,Angle,FArrowSize);
end;

function TTreeList.GetBackGround: TBitmap;
begin
  result:=FScrollBar.BackGround;
end;

function TTreeList.GetDownGliph: TBitmap;
begin
  result:=FScrollBar.SliderDownGliph;
end;

function TTreeList.GetGliph: TBitmap;
begin
  result:=FScrollBar.SliderGliph;
end;

function TTreeList.GetHighlightGliph: TBitmap;
begin
  result:=FScrollBar.SliderHighlightGliph;
end;

function TTreeList.GetImageMarigns: TMarigns;
begin
  result:=FScrollBar.ImageMarigns;
end;

function TTreeList.GetImages: TCustomImageList;
begin
  result:=FTreeView.Images;
end;

function TTreeList.GetItems: TTreeNodes;
begin
  result:=FTreeView.Items;
end;

function TTreeList.GetMinSliderWidth: integer;
begin
  result:=FScrollBar.MinSliderWidth;
end;

function TTreeList.GetSBColor: TColor;
begin
  result:=FScrollBar.Color;
end;

function TTreeList.GetSelection: TTreeNode;
begin
  result:=FTreeView.Selected;
end;

function TTreeList.GetSelectionChanged: TNotifyEvent;
begin
  result:=FTreeView.OnSelectionChanged;
end;

function TTreeList.GetSliderBackGround: TBitmap;
begin
  result:=FScrollBar.SliderBackGround;
end;

function TTreeList.GetSliderColor: TColor;
begin
  result:=FScrollBar.SliderColor;
end;

function TTreeList.GetSliderImageMarigns: TMarigns;
begin
  result:=FScrollBar.SliderImageMarigns;
end;

function TTreeList.GetSliderMarigns: TMarigns;
begin
  result:=FScrollBar.SliderMarigns;
end;

function TTreeList.GetTopItem: TTreeNode;
begin
  result:=FTreeView.TopItem;
end;

procedure TTreeList.SetAllowRootArrow(const AValue: boolean);
begin
  FAllowRootArrow:=AValue;
  Repaint;
end;

procedure TTreeList.SetBackGround(const AValue: TBitmap);
begin
  FScrollBar.BackGround:=AValue;
end;

procedure TTreeList.SetDownGliph(const AValue: TBitmap);
begin
  FScrollBar.SliderDownGliph:=AValue;
end;

procedure TTreeList.SetFont(const AValue: TFont);
begin
  FTreeView.Font.Assign(AValue);
  FTreeView.Repaint;
end;

procedure TTreeList.SetGliph(const AValue: TBitmap);
begin
  FScrollBar.SliderGliph:=AValue;
end;

procedure TTreeList.SetHighlightGliph(const AValue: TBitmap);
begin
  FScrollBar.SliderHighlightGliph:=AValue;
end;

procedure TTreeList.SetImageMarigns(const AValue: TMarigns);
begin
  FScrollBar.ImageMarigns:=AValue;
end;

procedure TTreeList.SetImages(const AValue: TCustomImageList);
begin
  FTreeView.Images:=AValue;
end;

procedure TTreeList.SetItems(const AValue: TTreeNodes);
begin
  FTreeView.Items:=AValue;
end;

procedure TTreeList.SetMinSliderWidth(const AValue: integer);
begin
  FScrollBar.MinSliderWidth:=AValue;
end;

procedure TTreeList.SetSBColor(const AValue: TColor);
begin
  FScrollBar.Color:=AValue;
end;

procedure TTreeList.SetSelectedFont(const AValue: TFont);
begin
  FSelectedFont.Assign(AValue);
  FTreeView.Repaint;
end;

procedure TTreeList.SetSelection(const AValue: TTreeNode);
begin
  FTreeView.Selected:=AValue;
end;

procedure TTreeList.SetSelectionChanged(const AValue: TNotifyEvent);
begin
  FTreeView.OnSelectionChanged:=AValue;
end;

procedure TTreeList.DoEnter;
begin
  inherited DoEnter;
  Repaint;
end;

procedure TTreeList.DoExit;
begin
  inherited DoExit;
  Repaint;
end;

procedure TTreeList.DoOnResize;
begin
  inherited DoOnResize;
  UpdateScroll;
end;

procedure TTreeList.MouseEnter;
begin
  inherited MouseEnter;
  FHover:=true;
  Repaint;
end;

procedure TTreeList.MouseLeave;
begin
  inherited MouseLeave;
  FHover:=false;
  Repaint;
end;

procedure TTreeList.UpdateScroll;
  function Max(i1, i2: integer): integer;
  begin
    if i1>i2 then
      Result:=i1
    else
      Result:=i2;
  end;

var
  aHeight,nPage,nMax:integer;
  LastVisibleNode: TTreeNode;
begin
  if (FTreeView=nil) or (FScrollBar=nil) then exit;
  FTreeView.Left:=0;
  FTreeView.Top:=0;
  FScrollBar.Top:=0;
  LastVisibleNode:=FTreeView.Items.GetLastExpandedSubNode;
  if LastVisibleNode=nil then
    aHeight:=0
  else
    aHeight:=LastVisibleNode.Top+LastVisibleNode.Height;
  if FOnUpdateScroll<>nil then FOnUpdateScroll(Self);
  FTreeView.Height:=Height;
  FScrollBar.Height:=Height;
  FScrollBar.Left:=Width-FScrollBar.Width;
  if FTreeView.Font.Size>=FSelectedFont.Size then
    FTreeView.Canvas.Font.Assign(FTreeView.Font)
  else
    FTreeView.Canvas.Font.Assign(FSelectedFont);
  FTreeView.DefaultItemHeight:=FTreeView.Canvas.TextHeight('gH')+2;
  nPage := Max(1,(aHeight-FScrollBar.Width)-FTreeView.DefaultItemHeight);
  nMax := Max(1,FTreeView.MaxScrollTop+nPage-1);
  if aHeight<=Height then
  begin
    FScrollBar.Visible:=false; //hide scrollbar
    FTreeView.Width:=FScrollBar.Left+FScrollBar.Width-FTreeView.Left;
  end else
  begin
    //maximal postion is total height of all items, but height of control
    FScrollBar.Count:=nMax-nPage;
    //show scrollbar
    FScrollBar.Visible:=true;
    FTreeView.Width:=FScrollBar.Left-FTreeView.Left;
  end;
end;

procedure TTreeList.TimerTimer(Sender: TObject);
begin
  if csDestroying in ComponentState then exit;
  if fExpanding then
    FTimer.Tag:=FTimer.Tag+{$IFDEF WIN32}3{$ELSE}1{$ENDIF}
  else
    FTimer.Tag:=FTimer.Tag-{$IFDEF WIN32}3{$ELSE}1{$ENDIF};
  DrawArrow(FTreeView,fNode,-FTimer.Tag/180*pi);
  if fExpanding then
    FTimer.Enabled:=FTimer.Tag<>90
  else
    FTimer.Enabled:=FTimer.Tag<>0;
  fCanDrawArrow:=not FTimer.Enabled;
end;

procedure TTreeList.TreeListCollapsing(Sender: TObject; Node: TTreeNode;
  var AllowCollapse: Boolean);
begin
  if csDestroying in ComponentState then exit;
  if (Node.Parent<>nil) or (FAllowRootArrow) then
  begin
    fCanDrawArrow:=false;
    AllowCollapse:=true;
    fnode:=Node;
    fExpanding:=false;
    FTimer.Tag:=90;
    FTimer.Enabled:=true;
  end else
    AllowCollapse:=false;
end;

procedure TTreeList.TreeListCollapsed(Sender: TObject; Node: TTreeNode);
begin
  if csDestroying in ComponentState then exit;
  UpdateScroll;
end;

procedure TTreeList.TreeListExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
begin
  if csLoading in ComponentState then exit;
  AllowExpansion:=true;
  if (Node.Parent<>nil) or (FAllowRootArrow) then
  begin
    fCanDrawArrow:=false;
    fnode:=Node;
    fExpanding:=True;
    FTimer.Tag:=0;
    FTimer.Enabled:=true;
  end;
end;

procedure TTreeList.TreeListExpanded(Sender: TObject; Node: TTreeNode);
begin
 UpdateScroll;
end;

procedure TTreeList.TreeListCustomDrawItem(Sender: TCustomTreeView;
  Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
var
  aRect:TRect;
  sz:integer;
  aBitmap:TCustomBitmap;
begin
  DefaultDraw:=false;
  Sender.Canvas.Brush.Color:=clWhite;
  Sender.Canvas.Brush.Style:=bsSolid;
  Sender.Canvas.FillRect(Node.DisplayRect(false));
  aRect:=Node.DisplayRect(true);
  aBitmap:=TCustomBitmap.Create;
  if FTreeView.Images<>nil then
    if cdsSelected in State then
    begin
      if Node.SelectedIndex<FTreeView.Images.Count then
        FTreeView.Images.GetBitmap(Node.SelectedIndex,aBitmap);
    end else
    begin
      if Node.ImageIndex<FTreeView.Images.Count then
        FTreeView.Images.GetBitmap(Node.ImageIndex,aBitmap);
    end;
  if aBitmap.BitmapHandleAllocated then
  if (Node.Parent<>nil) then
    Sender.Canvas.Draw(aRect.Left-aBitmap.Width-Sender.DefaultItemHeight,(aRect.Top+aRect.Bottom) div 2-aBitmap.Height div 2,aBitmap)
  else if not FAllowRootArrow then
    Sender.Canvas.Draw(aRect.Left-aBitmap.Width,(aRect.Top+aRect.Bottom) div 2-aBitmap.Height div 2,aBitmap);
  aBitmap.Free;
  Sender.Canvas.Font.Color:=Sender.Font.Color;
  sz:=Sender.Canvas.Font.Size;
  Sender.Canvas.Font.Size:=Sender.Font.Size;
  if cdsSelected in State then
  begin
    Sender.Canvas.Font.Color:=FSelectedFont.Color;
    Sender.Canvas.Font.Size:=FSelectedFont.Size;
  end;
  Sender.Canvas.TextRect(aRect,aRect.Left,aRect.Top+(aRect.Bottom- aRect.Top) div 2-Sender.Canvas.TextHeight(Node.Text) div 2,Node.Text);
  Sender.Canvas.Font.Size:=sz;
  if not (not fCanDrawArrow and (fNode=Node)) and (Node.Count>0) and ((Node.Parent<>nil) or FAllowRootArrow) then
    if Node.Expanded then
      DrawArrow(Sender,Node,-pi/2)
    else
      DrawArrow(Sender,Node,0);
end;

procedure TTreeList.TreeListCustomDraw(Sender: TCustomTreeView;
  const ARect: TRect; var DefaultDraw: Boolean);
var
  aCanvas:TCanvas; //control canvas
  lRect:TRect;//drawing rectangle
  i:integer;
begin
  DefaultDraw:=true;
  aCanvas:=TControlCanvas.Create; //get control canvas
  (aCanvas as TControlCanvas).Control:=self; //set drawing control
  aCanvas.Handle:={$IFDEF WIN32}GetWindowDC{$ELSE}GetDC{$ENDIF}(FTreeView.Handle); //and drawing context
  lRect:=FTreeView.BoundsRect;//drawing rect is a control rect
  lRect.Top:=lRect.Top-FTreeView.Top;
  lRect.Bottom:=lRect.Bottom-FTreeView.Top;
  lRect.Left:=lRect.Left-FTreeView.Left;
  lRect.Right:=lRect.Right-FTreeView.Left;
  aCanvas.Lock; //lock drawing to canvas
  aCanvas.Brush.Style:=bsClear; //set fill style as transparent
  aCanvas.Pen.Color:=Color;//set rect color to a control background color
  aCanvas.Rectangle(lRect); //draw single line rect
  DecRect(lRect); //decrease drawing rectangle size by one pixel and do same two times
  aCanvas.Rectangle(lRect);
  DecRect(lRect);
  aCanvas.Rectangle(lRect);
  DecRect(lRect,-2);
  for i:=1 to FBorderWidth do
  begin
    if FHover then
      if FTreeView.Focused then //if control focused
        aCanvas.Pen.Color:=FBorderSelectedLightColor //set blue border color
      else
        aCanvas.Pen.Color:=FBorderLightColor //set gray border color
    else
      if FTreeView.Focused then //if control focused
        aCanvas.Pen.Color:=FBorderSelectedColor //set blue border color
      else
        aCanvas.Pen.Color:=FBorderColor; //set gray border color
      aCanvas.Rectangle(lRect);
    DecRect(lRect);
  end;
  aCanvas.Unlock; //paint new canvas
  ReleaseDC(Handle,aCanvas.Handle); //release drawing context
  aCanvas.Free; //free memory from drawing surface class
end;

procedure TTreeList.ScrollBarScroll(Sender: TObject);
begin
  FTreeView.ScrolledTop:=round(FScrollBar.Position);
end;

procedure TTreeList.SetScrollBarWidth(Value: integer);
begin
  FScrollBar.Width:=Value;
end;

function TTreeList.GetScrollBarWidth: integer;
begin
  Result:=FScrollBar.Width;
end;

function TTreeList.GetFont: TFont;
begin
  result:=FTreeView.Font;
end;

function TTreeList.GetColor: TColor;
begin
  result:=FTreeView.Color;
end;

procedure TTreeList.SetColor(Value: TColor);
begin
  FTreeView.Color:=Value;
end;

procedure TTreeList.SetSliderColor(const AValue: TColor);
begin
  FScrollBar.SliderColor:=AValue;
end;

procedure TTreeList.SetSliderImageMarigns(const AValue: TMarigns);
begin
  FScrollBar.SliderImageMarigns:=AValue;
end;

procedure TTreeList.SetSliderMarigns(const AValue: TMarigns);
begin
  FScrollBar.SliderMarigns:=AValue;
end;

procedure TTreeList.SetTopItem(const AValue: TTreeNode);
begin
  FTreeView.TopItem:=AValue;
end;

constructor TTreeList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHover:=false;
  FAllowRootArrow:=true;
  FBorderColor:=clBlack;
  FBorderLightColor:=clDkGray;
  FBorderSelectedColor:=clGray;
  FBorderSelectedLightColor:=clLtGray;
  FBorderWidth:=1;
  Width:=100;
  Height:=150;
  FTreeView:=TMyTreeView.Create(self);
  FTreeView.Parent:=Self;
  FTreeView.ControlStyle := FTreeView.ControlStyle + [csNoDesignSelectable];
  FTreeView.FreeNotification(Self);
  FTreeView.Left:=0;
  FTreeView.Top:=0;
  FTreeView.Width:=Width;
  FTreeView.Height:=Height;
  FTreeView.Options:=[tvoAutoItemHeight, tvoKeepCollapsedNodes, tvoReadOnly,
                      tvoRightClickSelect, tvoRowSelect, tvoShowButtons,
                      tvoShowRoot, tvoToolTips];
  FTreeView.ScrollBars:=ssNone;
  FTreeView.ExpandSignType:=tvestArrow;
  FTreeView.ReadOnly:=True;
  FTreeView.RowSelect:=True;
  FTreeView.RightClickSelect:=True;
  FTreeView.ShowLines:=False;
  FTreeView.DoubleBuffered:=True;
  FTreeView.Visible:=true;
  FTreeView.OnCollapsed:=@TreeListCollapsed;
  FTreeView.OnCollapsing:=@TreeListCollapsing;
  FTreeView.OnExpanded:=@TreeListExpanded;
  FTreeView.OnExpanding:=@TreeListExpanding;
  FTreeView.OnCustomDrawItem:=@TreeListCustomDrawItem;
  FTreeView.OnCustomDraw:=@TreeListCustomDraw;
  FTreeView.OnMouseEnter:=@TreeListMouseEnter;
  FTreeView.OnMouseLeave:=@TreeListMouseLeave;
  FTreeView.OnEnter:=@TreeListEnter;
  FTreeView.OnExit:=@TreeListExit;
  FTreeView.OnMouseWheel:=@TreeListWhell;
  FTreeView.OnCustomSelection:=@TreeListCustomSelection;
  FScrollBar:=TMaskImageScrollBar.Create(self);
  FScrollBar.SetSubComponent(true);
  FScrollBar.ControlStyle := FScrollBar.ControlStyle + [csNoDesignSelectable];
  FScrollBar.FreeNotification(Self);
  FScrollBar.Name:='SubScrollBar';
  FScrollBar.Parent:=self;
  FScrollBar.Kind:=sbVertical;
  FScrollBar.Width:=10;
  FScrollBar.Height:=Height;
  FScrollBar.Top:=0;
  FScrollBar.Left:=Width-FScrollBar.Width;
  FScrollBar.Visible:=false;
  FScrollBar.OnScroll:=@ScrollBarScroll;
  FTimer:=TTimer.Create(self);
  FTimer.OnTimer:=@TimerTimer;
  FTimer.Interval:=2;
  FTimer.Enabled:=false;
  FArrowSize:=3;
  FTreeView.Font.Size:=14;
  FSelectedFont:=TFont.Create;
  FSelectedFont.Size:=14;
end;

destructor TTreeList.Destroy;
begin
  FTimer.Free;
  FScrollBar.Free;
  FTreeView.Free;
  FSelectedFont.Free;
end;

procedure TTreeList.Assign(Source: TPersistent);
begin
  //inherited Assign(Source);
  if not (Source is TTreeList) then exit;
  FTreeView.Assign((Source as TTreeList).FTreeView);
  FScrollBar.Assign((Source as TTreeList).FScrollBar);
  FBorderWidth:=(Source as TTreeList).FBorderWidth;
  FBorderColor:=(Source as TTreeList).FBorderColor;
  FBorderLightColor:=(Source as TTreeList).FBorderLightColor;
  FBorderSelectedColor:=(Source as TTreeList).FBorderSelectedColor;
  FBorderSelectedLightColor:=(Source as TTreeList).FBorderSelectedLightColor;
  FSelectedFont.Assign((Source as TTreeList).FSelectedFont);
  FAllowRootArrow:=(Source as TTreeList).FAllowRootArrow;
  FArrowSize:=(Source as TTreeList).FArrowSize;
  FArrowColor:=(Source as TTreeList).FArrowColor;
end;

procedure TTreeList.BeginUpdate;
begin
  FTreeView.BeginUpdate;
end;

procedure TTreeList.EndUpdate;
begin
  FTreeView.EndUpdate;
end;

procedure TTreeList.FullCollapse;
begin
  FTreeView.FullCollapse;
end;

procedure TTreeList.FullExpand;
begin
  FTreeView.FullExpand;
end;

{ TMyTreeView }

procedure TMyTreeView.DoSelectionChanged;
begin
  inherited DoSelectionChanged;
  if FOnCustomSelection<>nil then
    FOnCustomSelection(self);
end;

constructor TMyTreeView.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  Name:='SubTreeView';
  SetSubComponent(true);
end;

procedure TMyTreeView.Assign(Source: TPersistent);
begin
  if not (Source is TTreeView) then exit;
  Items.Assign((Source as TTreeView).Items);
  Color:=(Source as TTreeView).Color;
  Font.Assign((Source as TTreeView).Font);
  BackgroundColor:=(Source as TTreeView).BackgroundColor;
  ScrolledTop:=(Source as TMyTreeView).ScrolledTop;
end;

{ TComboBoxEx }

function TComboBoxEx.GetBorderColor: TColor;
begin
  result:=FEdit.FBorderColor;
end;

function TComboBoxEx.GetBackGround: TBitmap;
begin
  result:=FButton.BackGround;
end;

procedure TComboBoxEx.TreeSelectionUpdate(Sender: TObject);
begin
  if FOnUpdateScroll<>nil then FOnUpdateScroll(self);
end;

procedure TComboBoxEx.ComboDownList(Sender: TObject);
var
  Fl,Fr:TControl;
  p:TPoint;
  LastVisibleNode: TTreeNode;
begin
  FButton.SetFocus;
  Application.ProcessMessages;
  Fl:=self;
  Fr:=self;
  if (Owner is TWinControl) or Assigned(Parent) then
  begin
    if FLeftSibling<>nil then Fl:=FLeftSibling;
    if FRightSibling<>nil then Fr:=FRightSibling;
  end;
  with (FTreeListForm as TTreeListForm) do
  if Deactivated then
  begin
    if Fl=self then
      p:=self.ClientToScreen(Point(0,self.Height+1))
    else
      p:=self.Parent.ClientToScreen(Point(Fl.Left,self.Top+self.Height+1));
    Left:=p.x;
    Top:=p.y;
    if Fr=self then
      p:=self.ClientToScreen(Point(self.Width,self.Height+1))
    else
      p:=self.Parent.ClientToScreen(Point(Fr.Left+Fr.Width,self.Top+self.Height+1));
    Show;
    Width:=p.x-Left;
    LastVisibleNode:=self.Items.GetLastExpandedSubNode;
    if LastVisibleNode=nil then
      Height:=0
    else
      Height:=LastVisibleNode.Top+LastVisibleNode.Height;
    TreeSelectionUpdate(TreeList.FTreeView);
  end;
end;

procedure TComboBoxEx.TreeSelectionChanged(Sender: TObject);
begin
  if not (Sender is TTreeView) then exit;
  FTreeListForm.Close;
  if (Sender as TTreeView).Selected<>nil then
    FEdit.Text:=(Sender as TTreeView).Selected.Text
  else
    FEdit.Text:='';
  if FSelectionChanged<>nil then FSelectionChanged(self);
end;

function TComboBoxEx.GetArrowColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.ArrowColor;
end;

function TComboBoxEx.GetAllowRootArrow: boolean;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.AllowRootArrow;
end;

function TComboBoxEx.GetArrowSize: integer;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.ArrowSize;
end;

function TComboBoxEx.GetBorderLightColor: TColor;
begin
  result:=FEdit.FBorderLightColor;
end;

function TComboBoxEx.GetBorderSelectedColor: TColor;
begin
  result:=FEdit.FBorderSelectedColor;
end;

function TComboBoxEx.GetBorderSelectedLightColor: TColor;
begin
  result:=FEdit.FBorderSelectedLightColor;
end;

function TComboBoxEx.GetBorderWidth: integer;
begin
  result:=FEdit.FBorderWidth;
end;

function TComboBoxEx.GetButtonColor: TColor;
begin
  result:=FButton.Color;
end;

function TComboBoxEx.GetButtonFont: TFont;
begin
  result:=FButton.Font;
end;

function TComboBoxEx.GetCaption: TCaption;
begin
  result:=FButton.Caption;
end;

function TComboBoxEx.GetColor: TColor;
begin
  result:=FEdit.Color;
end;

function TComboBoxEx.GetComboHeight: integer;
begin
  result:=FTreeListForm.Height;
end;

function TComboBoxEx.GetComboListBorderColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.BorderColor;
end;

function TComboBoxEx.GetComboListBorderLightColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.BorderLightColor;
end;

function TComboBoxEx.GetComboListBorderSelectedColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.BorderSelectedColor;
end;

function TComboBoxEx.GetComboListBorderSelectedLightColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.BorderSelectedLightColor;
end;

function TComboBoxEx.GetComboListBorderWidth: integer;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.BorderWidth;
end;

function TComboBoxEx.GetComboListColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.Color;
end;

function TComboBoxEx.GetComboListFont: TFont;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.Font;
end;

function TComboBoxEx.GetDownGliph: TBitmap;
begin
  result:=FButton.DownGliph;
end;

function TComboBoxEx.GetDownHighlightGliph: TBitmap;
begin
  result:=FButton.DownHighlightGliph;
end;

function TComboBoxEx.GetFont: TFont;
begin
  result:=FEdit.Font;
end;

function TComboBoxEx.GetGliph: TBitmap;
begin
  result:=FButton.Gliph;
end;

function TComboBoxEx.GetHighlightGliph: TBitmap;
begin
  result:=FButton.HighlightGliph;
end;

function TComboBoxEx.GetImageMarigns: TMarigns;
begin
  result:=FButton.ImageMarigns;
end;

function TComboBoxEx.GetImages: TCustomImageList;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.Images;
end;

function TComboBoxEx.GetItems: TTreeNodes;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.Items;
end;

function TComboBoxEx.GetMinSliderWidth: integer;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.MinSliderWidth;
end;

function TComboBoxEx.GetPicture: TBitmap;
begin
  result:=FButton.Picture;
end;

function TComboBoxEx.GetSBColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.ScrollBarColor;
end;

function TComboBoxEx.GetScrollBarWidth: integer;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.ScrollBarWidth;
end;

function TComboBoxEx.GetSelectedFont: TFont;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.SelectedFont;
end;

function TComboBoxEx.GetSelection: TTreeNode;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.Selected;
end;

function TComboBoxEx.GetSliderBackGround: TBitmap;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.SliderBackGround;
end;

function TComboBoxEx.GetSliderColor: TColor;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.SliderColor;
end;

function TComboBoxEx.GetSliderImageMarigns: TMarigns;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.ImageMarigns;
end;

function TComboBoxEx.GetSliderMarigns: TMarigns;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList.SliderMarigns;
end;

function TComboBoxEx.GetText: TCaption;
begin
  result:=FEdit.Text;
end;

function TComboBoxEx.GetTreeList: TTreeList;
begin
  result:=(FTreeListForm as TTreeListForm).TreeList;
end;

procedure TComboBoxEx.SetAllowRootArrow(const AValue: boolean);
begin
  (FTreeListForm as TTreeListForm).TreeList.AllowRootArrow:=AValue;
end;

procedure TComboBoxEx.SetArrowColor(const AValue: TColor);
begin
  (FTreeListForm as TTreeListForm).TreeList.ArrowColor:=AValue;
end;

procedure TComboBoxEx.SetArrowSize(const AValue: integer);
begin
  (FTreeListForm as TTreeListForm).TreeList.ArrowSize:=AValue;
end;

procedure TComboBoxEx.SetBackGround(const AValue: TBitmap);
begin
  FButton.BackGround:=AValue;
end;

procedure TComboBoxEx.SetBorderColor(const AValue: TColor);
begin
  FEdit.FBorderColor:=AValue;
  FEdit.Repaint;
end;

procedure TComboBoxEx.SetBorderLightColor(const AValue: TColor);
begin
  FEdit.FBorderLightColor:=AValue;
  FEdit.Repaint;
end;

procedure TComboBoxEx.SetBorderSelectedColor(const AValue: TColor);
begin
  FEdit.FBorderSelectedColor:=AValue;
  FEdit.Repaint;
end;

procedure TComboBoxEx.SetBorderSelectedLightColor(const AValue: TColor);
begin
  FEdit.FBorderSelectedLightColor:=AValue;
  FEdit.Repaint;
end;

procedure TComboBoxEx.SetBorderWidth(const AValue: integer);
begin
  FEdit.FBorderWidth:=AValue;
  FEdit.Repaint;
end;

procedure TComboBoxEx.SetButtonColor(const AValue: TColor);
begin
  FButton.Color:=AValue;
end;

procedure TComboBoxEx.SetButtonFont(const AValue: TFont);
begin
  FButton.Font:=AValue;
end;

procedure TComboBoxEx.SetCaption(const AValue: TCaption);
begin
  FButton.Caption:=AValue;
end;

procedure TComboBoxEx.SetColor(const AValue: TColor);
begin
  FEdit.Color:=AValue;
end;

procedure TComboBoxEx.SetComboHeight(const AValue: integer);
begin
  FTreeListForm.Height:=AValue;
end;

procedure TComboBoxEx.SetComboListBorderColor(const AValue: TColor);
begin
  (FTreeListForm as TTreeListForm).TreeList.BorderColor:=AValue;
end;

procedure TComboBoxEx.SetComboListBorderLightColor(const AValue: TColor);
begin
  (FTreeListForm as TTreeListForm).TreeList.BorderLightColor:=AValue;
end;

procedure TComboBoxEx.SetComboListBorderSelectedColor(const AValue: TColor);
begin
  (FTreeListForm as TTreeListForm).TreeList.BorderSelectedColor:=AValue;
end;

procedure TComboBoxEx.SetComboListBorderSelectedLightColor(const AValue: TColor
  );
begin
  (FTreeListForm as TTreeListForm).TreeList.BorderSelectedLightColor:=AValue;
end;

procedure TComboBoxEx.SetComboListBorderWidth(const AValue: integer);
begin
  (FTreeListForm as TTreeListForm).TreeList.BorderWidth:=AValue;
end;

procedure TComboBoxEx.SetComboListColor(const AValue: TColor);
begin
  (FTreeListForm as TTreeListForm).TreeList.Color:=AValue;
end;

procedure TComboBoxEx.SetComboListFont(const AValue: TFont);
begin
  (FTreeListForm as TTreeListForm).TreeList.Font:=AValue;
end;

procedure TComboBoxEx.SetDownGliph(const AValue: TBitmap);
begin
  FButton.DownGliph:=AValue;
end;

procedure TComboBoxEx.SetDownHighlightGliph(const AValue: TBitmap);
begin
  FButton.DownHighlightGliph:=AValue;
end;

procedure TComboBoxEx.SetEnabled(const AValue: boolean);
begin
  FEnabled:=AValue;
  FEdit.Enabled:=FEnabled;
  FButton.Enabled:=FEnabled;
  Repaint;
end;

procedure TComboBoxEx.SetFont(const AValue: TFont);
begin
  FEdit.Font:=AValue;
end;

procedure TComboBoxEx.SetGliph(const AValue: TBitmap);
begin
  FButton.Gliph:=AValue;
end;

procedure TComboBoxEx.SetHighlightGliph(const AValue: TBitmap);
begin
  FButton.HighlightGliph:=AValue;
end;

procedure TComboBoxEx.SetImageMarigns(const AValue: TMarigns);
begin
  FButton.ImageMarigns:=AValue;
end;

procedure TComboBoxEx.SetImages(const AValue: TCustomImageList);
begin
  (FTreeListForm as TTreeListForm).TreeList.Images:=AValue;
end;

procedure TComboBoxEx.SetItems(const AValue: TTreeNodes);
begin
  (FTreeListForm as TTreeListForm).TreeList.Items:=AValue;
end;

procedure TComboBoxEx.SetMinSliderWidth(const AValue: integer);
begin
  (FTreeListForm as TTreeListForm).TreeList.MinSliderWidth:=AValue;
end;

procedure TComboBoxEx.SetPicture(const AValue: TBitmap);
begin
  FButton.Picture:=AValue;
end;

procedure TComboBoxEx.SetSBColor(const AValue: TColor);
begin
  (FTreeListForm as TTreeListForm).TreeList.ScrollBarColor:=AValue;
end;

procedure TComboBoxEx.SetScrollBarWidth(const AValue: integer);
begin
  (FTreeListForm as TTreeListForm).TreeList.ScrollBarWidth:=AValue;
end;

procedure TComboBoxEx.SetSelectedFont(const AValue: TFont);
begin
  (FTreeListForm as TTreeListForm).TreeList.SelectedFont:=AValue;
end;

procedure TComboBoxEx.SetSelection(const AValue: TTreeNode);
begin
  (FTreeListForm as TTreeListForm).TreeList.Selected:=AValue;
end;

procedure TComboBoxEx.SetSliderColor(const AValue: TColor);
begin
  (FTreeListForm as TTreeListForm).TreeList.SliderColor:=AValue;
end;

procedure TComboBoxEx.SetSliderImageMarigns(const AValue: TMarigns);
begin
  (FTreeListForm as TTreeListForm).TreeList.SliderImageMarigns:=AValue;
end;

procedure TComboBoxEx.SetSliderMarigns(const AValue: TMarigns);
begin
  (FTreeListForm as TTreeListForm).TreeList.SliderMarigns:=AValue;
end;

procedure TComboBoxEx.SetText(const AValue: TCaption);
begin
  FEdit.Text:=AValue;
end;

procedure TComboBoxEx.DoOnResize;
begin
  inherited DoOnResize;
  if (FEdit=nil) or (FButton=nil) then exit;
  FEdit.Height:=Height;
  FEdit.Width:=Width-Height;
  FButton.Width:=Height;
  FButton.Height:=Height;
  FButton.Left:=Width-Height+1;
end;

constructor TComboBoxEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width:=123;
  Height:=23;
  FEdit:=TEditEx.Create(self);
  FEdit.Parent:=Self;
  FEdit.SetSubComponent(true);
  FEdit.ControlStyle := FEdit.ControlStyle + [csNoDesignSelectable];
  FEdit.FreeNotification(Self);
  FEdit.AutoSize:=false;
  FEdit.Left:=0;
  FEdit.Top:=0;
  FEdit.Width:=Width-Height;
  FEdit.Height:=Height;
  FEdit.ReadOnly:=true;
  FEdit.Text:='';
  FEdit.Visible:=true;
  FEdit.OnClick:=@ComboDownList;
  FButton:=TMaskImageBtn.Create(self);
  FButton.Parent:=Self;
  FButton.SetSubComponent(true);
  FButton.ControlStyle := FButton.ControlStyle + [csNoDesignSelectable];
  FButton.FreeNotification(Self);
  FButton.Left:=Width-Height+1;
  FButton.Top:=0;
  FButton.Width:=Height;
  FButton.Height:=Height;
  FButton.Caption:='▼';
  FButton.Visible:=true;
  FButton.OnClick:=@ComboDownList;
  FLeftSibling:=nil;
  FRightSibling:=nil;
  FTreeListForm:=TTreeListForm.Create(nil);
  (FTreeListForm as TTreeListForm).TreeList.OnSelectionChanged:=@TreeSelectionChanged;
  (FTreeListForm as TTreeListForm).TreeList.OnUpdateScroll:=@TreeSelectionUpdate;
  FEnabled:=true;
end;

destructor TComboBoxEx.Destroy;
begin
  FTreeListForm.Free;
  FButton.Free;
  FEdit.Free;
end;

procedure TComboBoxEx.Assign(Source: TPersistent);
begin
//  inherited Assign(Source);
  if not (Source is TComboBoxEx) then exit;
  FButton.Assign((Source as TComboBoxEx).FButton);
  FEdit.Assign((Source as TComboBoxEx).FEdit);
  (FTreeListForm as TTreeListForm).TreeList.Assign(((Source as TComboBoxEx).FTreeListForm as TTreeListForm).TreeList);
end;

end.
